@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// Task Lua API
+// Taskwarrior Lua API
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -47,8 +47,8 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <iostream> // TODO Remove
 #include <algorithm>
+#include <iostream>
 #include "Context.h"
 #include "API.h"
 
@@ -58,7 +58,8 @@ Task* the_task = NULL;
 #ifdef HAVE_LIBLUA
 
 ////////////////////////////////////////////////////////////////////////////////
-// Returns a string representing the task version number, such as '1.9.0'.
+// Returns a string representing the taskwarrior version number, such as
+// '1.9.0'.
 static int api_task_version (lua_State* L)
 {
   lua_pushstring (L, PACKAGE_VERSION);
@@ -223,7 +224,8 @@ static int api_task_debug_message (lua_State* L)
 static int api_task_exit (lua_State* L)
 {
   // TODO Is this the correct exception?  How does the shell handle this?
-  throw std::string ("Exiting.");
+  std::cout << "Exiting." << std::endl;
+  exit (0); 
   return 0;
 }
 
@@ -612,14 +614,14 @@ bool API::callProgramHook (
 
   // Make call.
   if (lua_pcall (L, 0, 2, 0) != 0)
-    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1);
+    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1) + ".";
 
   // Call successful - get return values.
   if (!lua_isnumber (L, -2))
-    throw std::string ("Error: '") + function + "' did not return a success indicator";
+    throw std::string ("Error: '") + function + "' did not return a success indicator.";
 
   if (!lua_isstring (L, -1) && !lua_isnil (L, -1))
-    throw std::string ("Error: '") + function + "' did not return a message or nil";
+    throw std::string ("Error: '") + function + "' did not return a message or nil.";
 
   int rc              = lua_tointeger (L, -2);
   const char* message = lua_tostring  (L, -1);
@@ -684,17 +686,17 @@ bool API::callTaskHook (
 
   // Make call.
   if (lua_pcall (L, 1, 2, 0) != 0)
-    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1);
+    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1) + ".";
 
   // Hide the task.
   the_task = NULL;
 
   // Call successful - get return values.
   if (!lua_isnumber (L, -2))
-    throw std::string ("Error: '") + function + "' did not return a success indicator";
+    throw std::string ("Error: '") + function + "' did not return a success indicator.";
 
   if (!lua_isstring (L, -1) && !lua_isnil (L, -1))
-    throw std::string ("Error: '") + function + "' did not return a message or nil";
+    throw std::string ("Error: '") + function + "' did not return a message or nil.";
 
   int rc              = lua_tointeger (L, -2);
   const char* message = lua_tostring  (L, -1);
@@ -737,17 +739,17 @@ bool API::callFieldHook (
 
   // Make call.
   if (lua_pcall (L, 2, 3, 0) != 0)
-    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1);
+    throw std::string ("Error calling '") + function + "' - " + lua_tostring (L, -1) + ".";
 
   // Call successful - get return values.
   if (!lua_isstring (L, -3))
-    throw std::string ("Error: '") + function + "' did not return a modified value";
+    throw std::string ("Error: '") + function + "' did not return a modified value.";
 
   if (!lua_isnumber (L, -2))
-    throw std::string ("Error: '") + function + "' did not return a success indicator";
+    throw std::string ("Error: '") + function + "' did not return a success indicator.";
 
   if (!lua_isstring (L, -1) && !lua_isnil (L, -1))
-    throw std::string ("Error: '") + function + "' did not return a message or nil";
+    throw std::string ("Error: '") + function + "' did not return a message or nil.";
 
   const char* new_value = lua_tostring  (L, -3);
   int rc                = lua_tointeger (L, -2);
