@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -25,6 +25,7 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
+#include <algorithm>
 #include <sstream>
 #include <stdlib.h>
 #include <string.h>
@@ -50,6 +51,7 @@ static const char* internalNames[] =
   "limit",
   "status",
   "description",
+  "tags",
   // Note that annotations are not listed.
 };
 
@@ -63,6 +65,7 @@ static const char* modifiableNames[] =
   "recur",
   "until",
   "wait",
+  "depends",
 };
 
 // Synonyms on the same line.
@@ -252,10 +255,11 @@ bool Att::validNameValue (
   {
     std::string error = "Ambiguous attribute '" + name + "' - could be either of "; // TODO i18n
 
+    std::sort (matches.begin (), matches.end ());
     std::string combined;
     join (combined, ", ", matches);
 
-    throw error + combined;
+    throw error + combined + ".";
   }
 
   name = matches[0];
@@ -271,17 +275,18 @@ bool Att::validNameValue (
     autoComplete (mod, candidates, matches);
 
     if (matches.size () == 0)
-      throw std::string ("Unrecognized modifier '") + mod + "'";
+      throw std::string ("Unrecognized modifier '") + mod + "'.";
 
     else if (matches.size () != 1)
     {
       std::string error = "Ambiguous modifier '" + mod + "' - could be either of "; // TODO i18n
 
+      std::sort (matches.begin (), matches.end ());
       std::string combined;
       join (combined, ", ", matches);
       error += combined;
 
-      throw error + combined;
+      throw error + combined + ".";
     }
 
     mod = matches[0];
@@ -334,13 +339,45 @@ bool Att::validNameValue (
     //      modify 'value' here accordingly.
   }
 
+  // Dates can now be either a date, or a duration that is added as an offset
+  // to the current date.
   else if (name == "due"   ||
            name == "until" ||
            name == "wait")
   {
     // Validate and convert to epoch.
     if (value != "")
-      value = Date (value, context.config.get ("dateformat")).toEpochString ();
+    {
+      // Try parsing as a duration.  If unsuccessful, try again, as a date.
+      try
+      {
+        Date now;
+        Duration dur (value);
+
+        if (dur.negative ())
+          value = (now - (time_t)dur).toEpochString ();
+        else
+          value = (now + (time_t)dur).toEpochString ();
+      }
+
+      // If the date parsing failed, try parsing as a duration.  If successful,
+      // add the duration to the current date.  If unsuccessful, propagate the
+      // original date parse error.
+
+      // Try parsing as a date.  If unsuccessfull, throw.
+      catch (...)
+      {
+        try
+        {
+          value = Date (value, context.config.get ("dateformat")).toEpochString ();
+        }
+
+        catch (std::string& e)
+        {
+          throw e;
+        }
+      }
+    }
   }
 
   else if (name == "recur")
@@ -378,7 +415,7 @@ bool Att::validNameValue (
             "\" is not a valid status.  Use 'pending', 'completed', 'deleted', 'recurring' or 'waiting'.";
   }
 
-  else if (! validInternalName (name) &&
+  else if (! validInternalName   (name) &&
            ! validModifiableName (name))
     throw std::string ("'") + name + "' is not a recognized attribute.";
 
@@ -401,11 +438,11 @@ bool Att::validMod (const std::string& mod)
 std::string Att::type (const std::string& name) const
 {
   if (name == "due"   ||
+      name == "wait"  ||
       name == "until" ||
       name == "start" ||
       name == "entry" ||
-      name == "end"   ||
-      name == "wait")
+      name == "end")
     return "date";
 
   else if (name == "recur")
@@ -466,10 +503,10 @@ void Att::parse (Nibbler& n)
         if (validMod (mod))
           mMod = mod;
         else
-          throw std::string ("The name '") + mod + "' is not a valid modifier"; // TODO i18n
+          throw std::string ("The name '") + mod + "' is not a valid modifier."; // TODO i18n
       }
       else
-        throw std::string ("Missing . or : after modifier"); // TODO i18n
+        throw std::string ("Missing . or : after modifier."); // TODO i18n
     }
 
     if (n.skip (':'))
@@ -483,10 +520,10 @@ void Att::parse (Nibbler& n)
       }
     }
     else
-      throw std::string ("Missing : after attribute name"); // TODO i18n
+      throw std::string ("Missing : after attribute name."); // TODO i18n
   }
   else
-    throw std::string ("Missing : after attribute name"); // TODO i18n
+    throw std::string ("Missing : after attribute name."); // TODO i18n
 
 /* TODO This might be too slow to include.  Test this assumption.
   validNameValue (mName, mMod, mValue);
@@ -505,16 +542,17 @@ bool Att::match (const Att& other) const
   if (mMod == "")
   {
     // Exact matches on dates should only compare m/d/y, not h:m:s.  This allows
-    // Comapisons like "task list due:today" (bug #405).
+    // comparisons like "task list due:today" (bug #405).
     std::string which = type (mName);
     if (which == "date")
     {
+      if (other.mValue == "")
+        return false;
+
       Date left (mValue);
       Date right (other.mValue);
 
-      if (left.year ()  != right.year ()  ||
-          left.month () != right.month () ||
-          left.day ()   != right.day ())
+      if (! left.sameDay (right))
         return false;
     }
     else
@@ -718,7 +756,7 @@ std::string Att::composeF4 () const
 void Att::mod (const std::string& input)
 {
   if (input != "" && !validMod (input))
-    throw std::string ("The name '") + input + "' is not a valid modifier"; // TODO i18n
+    throw std::string ("The name '") + input + "' is not a valid modifier."; // TODO i18n
 
   mMod = input;
 }
