@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -125,8 +125,6 @@ void Context::initialize ()
   if (locale != "")
     stringtable.load (location.data + "/strings." + locale);
 
-  // TODO Handle "--version, -v" right here?
-
   // init TDB.
   tdb.clear ();
   std::vector <std::string> all;
@@ -163,18 +161,19 @@ int Context::run ()
   if (config.getBoolean ("debug"))
     foreach (d, debugMessages)
       if (config.getBoolean ("color") || config.getBoolean ("_forcecolor"))
-        std::cout << colorizeDebug (*d) << std::endl;
+        std::cout << colorizeDebug (*d) << "\n";
       else
-        std::cout << *d << std::endl;
+        std::cout << *d << "\n";
   hooks.trigger ("post-debug");
 
   // Dump all headers.
   hooks.trigger ("pre-header");
-  foreach (h, headers)
-    if (config.getBoolean ("color") || config.getBoolean ("_forcecolor"))
-      std::cout << colorizeHeader (*h) << std::endl;
-    else
-      std::cout << *h << std::endl;
+  if (config.getBoolean ("verbose"))
+    foreach (h, headers)
+      if (config.getBoolean ("color") || config.getBoolean ("_forcecolor"))
+        std::cout << colorizeHeader (*h) << "\n";
+      else
+        std::cout << *h << "\n";
   hooks.trigger ("post-header");
 
   // Dump the report output.
@@ -184,11 +183,12 @@ int Context::run ()
 
   // Dump all footnotes.
   hooks.trigger ("pre-footnote");
-  foreach (f, footnotes)
-    if (config.getBoolean ("color") || config.getBoolean ("_forcecolor"))
-      std::cout << colorizeFootnote (*f) << std::endl;
-    else
-      std::cout << *f << std::endl;
+  if (config.getBoolean ("verbose"))
+    foreach (f, footnotes)
+      if (config.getBoolean ("color") || config.getBoolean ("_forcecolor"))
+        std::cout << colorizeFootnote (*f) << "\n";
+      else
+        std::cout << *f << "\n";
   hooks.trigger ("post-footnote");
 
   hooks.trigger ("pre-exit");
@@ -233,6 +233,7 @@ int Context::dispatch (std::string &out)
   else if (cmd.command == "stop")             { rc = handleStop                  (out); }
   else if (cmd.command == "export.csv")       { rc = handleExportCSV             (out); }
   else if (cmd.command == "export.ical")      { rc = handleExportiCal            (out); }
+  else if (cmd.command == "export.yaml")      { rc = handleExportYAML            (out); }
   else if (cmd.command == "import")           { rc = handleImport                (out); }
   else if (cmd.command == "duplicate")        { rc = handleDuplicate             (out); }
   else if (cmd.command == "edit")             { rc = handleEdit                  (out); }
@@ -240,19 +241,24 @@ int Context::dispatch (std::string &out)
   else if (cmd.command == "shell")            {      handleShell                 (   ); }
 #endif
   else if (cmd.command == "undo")             {      handleUndo                  (   ); }
-  else if (cmd.command == "_merge")           {      handleMerge                 (out); }
+  else if (cmd.command == "merge")            { tdb.gc ();
+                                                     handleMerge                 (out); }
+  else if (cmd.command == "push")             {      handlePush                  (out); }
+  else if (cmd.command == "pull")             {      handlePull                  (out); }
+  else if (cmd.command == "diagnostics")      {      handleDiagnostics           (out); }
   else if (cmd.command == "_projects")        { rc = handleCompletionProjects    (out); }
   else if (cmd.command == "_tags")            { rc = handleCompletionTags        (out); }
   else if (cmd.command == "_commands")        { rc = handleCompletionCommands    (out); }
   else if (cmd.command == "_ids")             { rc = handleCompletionIDs         (out); }
   else if (cmd.command == "_config")          { rc = handleCompletionConfig      (out); }
   else if (cmd.command == "_version")         { rc = handleCompletionVersion     (out); }
+  else if (cmd.command == "_urgency")         { rc = handleUrgency               (out); }
   else if (cmd.command == "" &&
            sequence.size ())                  { rc = handleModify                (out); }
 
-  // Command that display IDs and therefore need TDB::gc first.
-  else if (cmd.command == "next")             { if (!inShadow) tdb.gc (); rc = handleReportNext (out); }
-  else if (cmd.validCustom (cmd.command))     { if (!inShadow) tdb.gc (); rc = handleCustomReport  (cmd.command, out); }
+  // Commands that display IDs and therefore need TDB::gc first.
+  else if (cmd.validCustom (cmd.command))     { if (!inShadow) tdb.gc ();
+                                                rc = handleCustomReport (cmd.command, out); }
 
   // If the command is not recognized, display usage.
   else                                        { hooks.trigger ("pre-usage-command");
@@ -320,7 +326,7 @@ void Context::shadow ()
 
     // Optionally display a notification that the shadow file was updated.
     if (config.getBoolean ("shadow.notify"))
-      footnote (std::string ("[Shadow file '") + shadowFile.data + "' updated]");
+      footnote (std::string ("[Shadow file '") + shadowFile.data + "' updated.]");
 
     inShadow = false;
   }
@@ -368,6 +374,45 @@ void Context::disallowModification () const
         + "' command does not allow further modification of a task.";
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Takes a vector of args (foo, rc.name:value, bar), extracts any rc.name:value
+// args and sets the name/value in context.config, returning only the plain args
+// (foo, bar) as output.
+void Context::applyOverrides (
+  const std::vector <std::string>& input,
+  std::vector <std::string>& output)
+{
+  bool foundTerminator = false;
+  foreach (in, input)
+  {
+    if (*in == "--")
+    {
+      foundTerminator = true;
+      output.push_back (*in);
+    }
+    else if (!foundTerminator && in->substr (0, 3) == "rc.")
+    {
+      std::string name;
+      std::string value;
+      Nibbler n (*in);
+      if (n.getLiteral ("rc.")         &&  // rc.
+          n.getUntilOneOf (":=", name) &&  //    xxx
+          n.skipN (1))                     //       :
+      {
+        n.getUntilEOS (value);  // Don't care if it's blank.
+
+        config.set (name, value);
+        var_overrides += " " + *in;
+        footnote ("Configuration override " + in->substr (3));
+      }
+      else
+        footnote ("Problem with override: " + *in);
+    }
+    else
+      output.push_back (*in);
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void Context::loadCorrectConfigFile ()
 {
@@ -418,8 +463,8 @@ void Context::loadCorrectConfigFile ()
   {
     if (*arg == "--")
       break;
-    else if (arg->substr (0, 17) == "rc.data.location:" ||
-             arg->substr (0, 17) == "rc.data.location=")
+    else if (arg->substr (0, 16) == "rc.data.location" &&
+             ((*arg)[16] == ':' || (*arg)[16] == '='))
     {
       data = Directory (arg->substr (17));
       header ("Using alternate data.location " + data.data); // TODO i18n
@@ -430,60 +475,23 @@ void Context::loadCorrectConfigFile ()
   // Do we need to create a default rc?
   if (! rc.exists ())
   {
-    if (confirm ("A configuration file could not be found in " // TODO i18n
-               + home
-               + "\n\n"
-               + "Would you like a sample "
-               + rc.data
-               + " created, so task can proceed?"))
-    {
-      config.createDefaultRC (rc, data);
-    }
-    else
+    if (!confirm ("A configuration file could not be found in " // TODO i18n
+                + home
+                + "\n\n"
+                + "Would you like a sample "
+                + rc.data
+                + " created, so taskwarrior can proceed?"))
       throw std::string ("Cannot proceed without rc file.");
+
+    config.createDefaultRC (rc, data);
   }
 
   // Create data location, if necessary.
   config.createDefaultData (data);
 
-  // TODO find out why this was done twice - see tw #355
-  // Load rc file.
-  //config.clear ();       // Dump current values.
-  //config.setDefaults (); // Add in the custom reports.
-  //config.load (rc);      // Load new file.
-
-  // Apply overrides of type: "rc.name:value", or "rc.name=value".
+  // Apply rc overrides.
   std::vector <std::string> filtered;
-  bool foundTerminator = false;
-  foreach (arg, args)
-  {
-    if (*arg == "--")
-    {
-      foundTerminator = true;
-      filtered.push_back (*arg);
-    }
-    else if (!foundTerminator &&
-             arg->substr (0, 3) == "rc.")
-    {
-      std::string name;
-      std::string value;
-      Nibbler n (*arg);
-      if (n.getUntil ('.', name)       &&
-          n.skip ('.')                 &&
-          n.getUntilOneOf (":=", name) &&
-          n.skipN (1)                  &&
-          n.getUntilEOS (value))
-      {
-        config.set (name, value);
-        var_overrides += " " + *arg;
-        footnote (std::string ("Configuration override ") +  // TODO i18n
-                  arg->substr (3));
-      }
-    }
-    else
-      filtered.push_back (*arg);
-  }
-
+  applyOverrides (args, filtered);
   args = filtered;
 }
 
@@ -592,6 +600,18 @@ void Context::parse (
         tagRemovals.push_back (arg->substr (1));
       }
 
+      // Substitution of description and/or annotation text.
+      else if (parseSubst.valid (*arg))
+      {
+        if (foundSequence)
+          foundSomethingAfterSequence = true;
+
+        foundNonSequence = true;
+
+        debug ("parse subst '" + *arg + "'");
+        parseSubst.parse (*arg);
+      }
+
       // Atributes - name[.mod]:[value]
       else if (attribute.valid (*arg))
       {
@@ -638,18 +658,6 @@ void Context::parse (
         }
       }
 
-      // Substitution of description and/or annotation text.
-      else if (parseSubst.valid (*arg))
-      {
-        if (foundSequence)
-          foundSomethingAfterSequence = true;
-
-        foundNonSequence = true;
-
-        debug ("parse subst '" + *arg + "'");
-        parseSubst.parse (*arg);
-      }
-
       // It might be a command if one has not already been found.
       else if (parseCmd.command == "" &&
                parseCmd.valid (*arg))
@@ -749,7 +757,7 @@ void Context::parse (
       else
         throw stringtable.get (
           CMD_MISSING,
-          "You must specify a command, or a task ID to modify");
+          "You must specify a command, or a task ID to modify.");
     }
 
     // If the command "task 123" is entered, but with no modifier arguments,
@@ -757,7 +765,7 @@ void Context::parse (
     else if (!foundNonSequence &&
              (parseTask.id != 0 || parseSequence.size () != 0))
     {
-      std::cout << "No command - assuming 'info'." << std::endl;
+      std::cout << "No command - assuming 'info'.\n";
       parseCmd.command = "info";
     }
   }
@@ -833,11 +841,8 @@ void Context::autoFilter (Att& a, Filter& f)
   {
   }
 
-  // The mechanism for filtering on tags is +/-<tag>.
-  // Do not handle here - see below.
-  else if (a.name () == "tags")
-  {
-  }
+  // Note: Tags are handled via the +/-<tag> syntax, but also via attribute
+  // modifiers.
 
   // Generic attribute matching.
   else
