@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
@@ -24,7 +24,6 @@
 //     USA
 //
 ////////////////////////////////////////////////////////////////////////////////
-#include <iostream>
 #include <iomanip>
 #include <sstream>
 #include <time.h>
@@ -82,7 +81,7 @@ Date::Date (const int m,  const int d,  const int y,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-Date::Date (const std::string& mdy, const std::string& format /* = "m/d/Y" */)
+Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
 {
   int month  = 0;
   int day    = 0;
@@ -92,105 +91,109 @@ Date::Date (const std::string& mdy, const std::string& format /* = "m/d/Y" */)
   int second = 0;
 
   // Perhaps it is an epoch date, in string form?
-  if (isEpoch (mdy))
+  if (isEpoch (input))
     return;
 
   // Before parsing according to "format", perhaps this is a relative date?
-  if (isRelativeDate (mdy))
+  if (isRelativeDate (input))
     return;
 
-  unsigned int i = 0; // Index into mdy.
+  unsigned int i = 0; // Index into input.
 
+  // Format may include: mMdDyYVaAbBhHNS
+  //
+  // Note that the format should never include T or Z, as that interferes with
+  // the potential parsing for ISO dates constructed from the above format.
   for (unsigned int f = 0; f < format.length (); ++f)
   {
     switch (format[f])
     {
     // Single or double digit.
     case 'm':
-      if (i >= mdy.length () ||
-          ! isdigit (mdy[i]))
+      if (i >= input.length () ||
+          ! isdigit (input[i]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (m).";
+        throw std::string ("\"") + input + "\" is not a valid date (m).";
       }
 
-      if (i + 1 < mdy.length ()                    &&
-          (mdy[i + 0] == '0' || mdy[i + 0] == '1') &&
-          isdigit (mdy[i + 1]))
+      if (i + 1 < input.length ()                    &&
+          (input[i + 0] == '0' || input[i + 0] == '1') &&
+          isdigit (input[i + 1]))
       {
-        month = atoi (mdy.substr (i, 2).c_str ());
+        month = atoi (input.substr (i, 2).c_str ());
         i += 2;
       }
       else
       {
-        month = atoi (mdy.substr (i, 1).c_str ());
+        month = atoi (input.substr (i, 1).c_str ());
         ++i;
       }
       break;
 
     case 'd':
-      if (i >= mdy.length () ||
-          ! isdigit (mdy[i]))
+      if (i >= input.length () ||
+          ! isdigit (input[i]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (d).";
+        throw std::string ("\"") + input + "\" is not a valid date (d).";
       }
 
-      if (i + 1 < mdy.length ()                                                              &&
-          (mdy[i + 0] == '0' || mdy[i + 0] == '1' || mdy[i + 0] == '2' || mdy[i + 0] == '3') &&
-          isdigit (mdy[i + 1]))
+      if (i + 1 < input.length ()                                                                    &&
+          (input[i + 0] == '0' || input[i + 0] == '1' || input[i + 0] == '2' || input[i + 0] == '3') &&
+          isdigit (input[i + 1]))
       {
-        day = atoi (mdy.substr (i, 2).c_str ());
+        day = atoi (input.substr (i, 2).c_str ());
         i += 2;
       }
       else
       {
-        day = atoi (mdy.substr (i, 1).c_str ());
+        day = atoi (input.substr (i, 1).c_str ());
         ++i;
       }
       break;
 
     // Double digit.
     case 'y':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (y).";
+        throw std::string ("\"") + input + "\" is not a valid date (y).";
       }
 
-      year = atoi (mdy.substr (i, 2).c_str ()) + 2000;
+      year = atoi (input.substr (i, 2).c_str ()) + 2000;
       i += 2;
       break;
 
     case 'M':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (M).";
+        throw std::string ("\"") + input + "\" is not a valid date (M).";
       }
 
-      month = atoi (mdy.substr (i, 2).c_str ());
+      month = atoi (input.substr (i, 2).c_str ());
       i += 2;
       break;
 
     case 'D':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (D).";
+        throw std::string ("\"") + input + "\" is not a valid date (D).";
       }
 
-      day = atoi (mdy.substr (i, 2).c_str ());
+      day = atoi (input.substr (i, 2).c_str ());
       i += 2;
       break;
 
     case 'V':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (V).";
+        throw std::string ("\"") + input + "\" is not a valid date (V).";
       }
 
       i += 2;
@@ -198,134 +201,134 @@ Date::Date (const std::string& mdy, const std::string& format /* = "m/d/Y" */)
 
     // Quadruple digit.
     case 'Y':
-      if (i + 3 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]) ||
-          ! isdigit (mdy[i + 2]) ||
-          ! isdigit (mdy[i + 3]))
+      if (i + 3 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]) ||
+          ! isdigit (input[i + 2]) ||
+          ! isdigit (input[i + 3]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (Y).";
+        throw std::string ("\"") + input + "\" is not a valid date (Y).";
       }
 
-      year = atoi (mdy.substr (i, 4).c_str ());
+      year = atoi (input.substr (i, 4).c_str ());
       i += 4;
       break;
 
     // Short names with 3 characters
     case 'a':
-      if (i + 2 >= mdy.length () ||
-          isdigit (mdy[i + 0]) ||
-          isdigit (mdy[i + 1]) ||
-          isdigit (mdy[i + 2]))
+      if (i + 2 >= input.length () ||
+          isdigit (input[i + 0]) ||
+          isdigit (input[i + 1]) ||
+          isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (a).";
+        throw std::string ("\"") + input + "\" is not a valid date (a).";
       }
 
       i += 3;
       break;
 
     case 'b':
-      if (i + 2 >= mdy.length () ||
-          isdigit (mdy[i + 0]) ||
-          isdigit (mdy[i + 1]) ||
-          isdigit (mdy[i + 2]))
+      if (i + 2 >= input.length () ||
+          isdigit (input[i + 0]) ||
+          isdigit (input[i + 1]) ||
+          isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (b).";
+        throw std::string ("\"") + input + "\" is not a valid date (b).";
       }
 
-      month = Date::monthOfYear (mdy.substr (i, 3).c_str());
+      month = Date::monthOfYear (input.substr (i, 3).c_str());
       i += 3;
       break;
 
     // Long names
     case 'A':
-      if (i + 2 >= mdy.length () ||
-          isdigit (mdy[i + 0]) ||
-          isdigit (mdy[i + 1]) ||
-          isdigit (mdy[i + 2]))
+      if (i + 2 >= input.length () ||
+          isdigit (input[i + 0]) ||
+          isdigit (input[i + 1]) ||
+          isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (A).";
+        throw std::string ("\"") + input + "\" is not a valid date (A).";
       }
 
-      i += Date::dayName( Date::dayOfWeek (mdy.substr (i, 3).c_str()) ).size();
+      i += Date::dayName( Date::dayOfWeek (input.substr (i, 3).c_str()) ).size();
       break;
 
     case 'B':
-      if (i + 2 >= mdy.length () ||
-          isdigit (mdy[i + 0]) ||
-          isdigit (mdy[i + 1]) ||
-          isdigit (mdy[i + 2]))
+      if (i + 2 >= input.length () ||
+          isdigit (input[i + 0]) ||
+          isdigit (input[i + 1]) ||
+          isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (B).";
+        throw std::string ("\"") + input + "\" is not a valid date (B).";
       }
 
-      month = Date::monthOfYear (mdy.substr (i, 3).c_str());
+      month = Date::monthOfYear (input.substr (i, 3).c_str());
       i += Date::monthName(month).size();
       break;
 
     // Single or double digit.
     case 'h':
-      if (i >= mdy.length () ||
-          ! isdigit (mdy[i]))
+      if (i >= input.length () ||
+          ! isdigit (input[i]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (h).";
+        throw std::string ("\"") + input + "\" is not a valid date (h).";
       }
 
-      if (i + 1 < mdy.length ()                    &&
-          (mdy[i + 0] == '0' || mdy[i + 0] == '1' || mdy[i + 0] == '2') &&
-          isdigit (mdy[i + 1]))
+      if (i + 1 < input.length ()                    &&
+          (input[i + 0] == '0' || input[i + 0] == '1' || input[i + 0] == '2') &&
+          isdigit (input[i + 1]))
       {
-        hour = atoi (mdy.substr (i, 2).c_str ());
+        hour = atoi (input.substr (i, 2).c_str ());
         i += 2;
       }
       else
       {
-        hour = atoi (mdy.substr (i, 1).c_str ());
+        hour = atoi (input.substr (i, 1).c_str ());
         ++i;
       }
       break;
 
     case 'H':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (H).";
+        throw std::string ("\"") + input + "\" is not a valid date (H).";
       }
 
-      hour = atoi (mdy.substr (i, 2).c_str ());
+      hour = atoi (input.substr (i, 2).c_str ());
       i += 2;
       break;
 
     case 'N':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (N).";
+        throw std::string ("\"") + input + "\" is not a valid date (N).";
       }
 
-      minute = atoi (mdy.substr (i, 2).c_str ());
+      minute = atoi (input.substr (i, 2).c_str ());
       i += 2;
       break;
 
     case 'S':
-      if (i + 1 >= mdy.length () ||
-          ! isdigit (mdy[i + 0]) ||
-          ! isdigit (mdy[i + 1]))
+      if (i + 1 >= input.length () ||
+          ! isdigit (input[i + 0]) ||
+          ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (S).";
+        throw std::string ("\"") + input + "\" is not a valid date (S).";
       }
 
-      second = atoi (mdy.substr (i, 2).c_str ());
+      second = atoi (input.substr (i, 2).c_str ());
       i += 2;
       break;
 
     default:
-      if (i >= mdy.length () ||
-          mdy[i] != format[f])
+      if (i >= input.length () ||
+          input[i] != format[f])
       {
-        throw std::string ("\"") + mdy + "\" is not a valid date (DEFAULT).";
+        throw std::string ("\"") + input + "\" is not a valid date (DEFAULT).";
       }
       ++i;
       break;
@@ -340,11 +343,11 @@ Date::Date (const std::string& mdy, const std::string& format /* = "m/d/Y" */)
     year = default_year->tm_year + 1900;
   }
 
-  if (i < mdy.length ())
-    throw std::string ("\"") + mdy + "\" is not a valid date in " + format + " format.";
+  if (i < input.length ())
+    throw std::string ("\"") + input + "\" is not a valid date in " + format + " format.";
 
   if (!valid (month, day, year))
-    throw std::string ("\"") + mdy + "\" is not a valid date (VALID).";
+    throw std::string ("\"") + input + "\" is not a valid date (VALID).";
 
   // Convert to epoch.
   struct tm t = {0};
@@ -457,19 +460,6 @@ const std::string Date::toString (const std::string& format /*= "m/d/Y" */) cons
   return formatted;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-const std::string Date::toStringWithTime (const std::string& format /*= "m/d/Y" */) const
-{
-  // Format as above.
-  std::string formatted = toString (format);
-
-  char buffer[12];
-  sprintf (buffer, " %d:%02d:%02d", hour (), minute (), second ());
-  formatted += buffer;
-
-  return formatted;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::valid (const std::string& input, const std::string& format)
 {
@@ -817,6 +807,12 @@ bool Date::sameYear (const Date& rhs)
   return false;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+Date Date::operator- (const int delta)
+{
+  return Date (mT - delta);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Date Date::operator+ (const int delta)
 {
@@ -846,8 +842,9 @@ time_t Date::operator- (const Date& rhs)
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::isEpoch (const std::string& input)
 {
-  if (digitsOnly (input) &&
-      input.length () > 8)
+  if (digitsOnly (input)    &&
+      input.length () >   8 &&
+      input.length () <= 10 )
   {
     mT = (time_t) atoi (input.c_str ());
     return true;
@@ -871,6 +868,7 @@ bool Date::isEpoch (const std::string& input)
 //   eow         (end of week)
 //   eom         (end of month)
 //   eoy         (end of year)
+//   now
 bool Date::isRelativeDate (const std::string& input)
 {
   std::string in (lowerCase (input));
@@ -888,8 +886,15 @@ bool Date::isRelativeDate (const std::string& input)
   supported.push_back ("tomorrow");
   supported.push_back ("yesterday");
   supported.push_back ("eow");
+  supported.push_back ("eoww");
+  supported.push_back ("eocw");
   supported.push_back ("eom");
   supported.push_back ("eoy");
+  supported.push_back ("sow");
+  supported.push_back ("soww");
+  supported.push_back ("socw");
+  supported.push_back ("som");
+  supported.push_back ("soy");
   supported.push_back ("goodfriday");
   supported.push_back ("easter");
   supported.push_back ("eastermonday");
@@ -897,6 +902,7 @@ bool Date::isRelativeDate (const std::string& input)
   supported.push_back ("pentecost");
   supported.push_back ("midsommar");
   supported.push_back ("midsommarafton");
+  supported.push_back ("now");
 
   std::vector <std::string> matches;
   if (autoComplete (in, supported, matches) == 1)
@@ -906,11 +912,25 @@ bool Date::isRelativeDate (const std::string& input)
     // If day name.
     int dow;
     if ((dow = Date::dayOfWeek (found)) != -1 ||
-        found == "eow")
+        found == "eow"  ||
+        found == "eoww" ||
+        found == "eocw" ||
+        found == "sow"  ||
+        found == "soww" ||
+        found == "socw")
     {
-      if (found == "eow")
+      if (found == "eow" || found == "eoww")
         dow = 5;
 
+      if (found == "eocw")
+        dow = (Date::dayOfWeek (context.config.get ("weekstart")) + 6) % 7;
+
+      if (found == "sow" || found == "soww")
+        dow = 1;
+
+      if (found == "socw")
+        dow = Date::dayOfWeek (context.config.get ("weekstart"));
+
       if (today.dayOfWeek () >= dow)
         today += (dow - today.dayOfWeek () + 7) * 86400;
       else
@@ -961,6 +981,25 @@ bool Date::isRelativeDate (const std::string& input)
       mT = then.mT;
       return true;
     }
+    else if (found == "som")
+    {
+      int m = today.month () + 1;
+      int y = today.year ();
+      if (m > 12)
+      {
+        m -=12;
+        y++;
+      }
+      Date then (m, 1, y);
+      mT = then.mT;
+      return true;
+    }
+    else if (found == "soy")
+    {
+      Date then (1, 1, today.year () + 1);
+      mT = then.mT;
+      return true;
+    }
     else if (found == "goodfriday")
     {
       Date then (Date::easter(today.year()));
@@ -1015,6 +1054,11 @@ bool Date::isRelativeDate (const std::string& input)
         }
       }
     }
+    else if (found == "now")
+    {
+      mT = time (NULL);
+      return true;
+    }
   }
 
   // Support "21st" to indicate the next date that is the 21st day.
