@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -132,7 +132,7 @@ void Filter::applySequence (std::vector<Task>& all, Sequence& sequence)
     std::vector <int> right;
     listDiff (filteredSequence, (std::vector <int>&)sequence, left, right);
     if (left.size ())
-      throw std::string ("Sequence filtering error - please report this error to support@taskwarrior.org");
+      throw std::string ("Sequence filtering error - please report this error to support@taskwarrior.org.");
 
     if (right.size ())
     {
@@ -144,7 +144,7 @@ void Filter::applySequence (std::vector<Task>& all, Sequence& sequence)
       foreach (r, right)
         out << " " << *r;
 
-      out << " not found";
+      out << " not found.";
       throw out.str ();
     }
   }
