@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -52,9 +52,9 @@
 #define DEBUG_OUTPUT 0
 
 #if DEBUG_OUTPUT > 0
-  #define DEBUG_STR(str)       std::cout << "DEBUG: " << str << std::endl; std::cout.flush()
+  #define DEBUG_STR(str)       std::cout << "DEBUG: " << str << "\n"; std::cout.flush()
   #define DEBUG_STR_PART(str)  std::cout << "DEBUG: " << str; std::cout.flush()
-  #define DEBUG_STR_END(str)   std::cout << str << std::endl; std::cout.flush()
+  #define DEBUG_STR_END(str)   std::cout << str << "\n"; std::cout.flush()
 #else
   #define DEBUG_STR(str)
   #define DEBUG_STR_PART(str)
@@ -69,39 +69,44 @@ void readTaskmods (std::vector <std::string> &input,
                    std::vector <std::string>::iterator &start,
                    std::list<Taskmod> &list)
 {
-  std::string line;
-  Taskmod tmod_tmp;
+  std::string  line;
+  Taskmod      tmod_tmp;
 
-  DEBUG_STR("reading taskmods from file: ");
+  DEBUG_STR ("reading taskmods from file: ");
 
-  for (;start != input.end(); ++start) {
+  for ( ; start != input.end (); ++start)
+  {
     line = *start;
 
-    if (line.substr (0, 4) == "time") {
-      std::stringstream stream(line.substr(5));
+    if (line.substr (0, 4) == "time")
+    {
+      std::stringstream stream (line.substr (5));
       long ts;
       stream >> ts;
 
-      if (stream.fail()) {
-        throw std::string ("Failed to convert \"" + stream.str() + "\" to integer: " + tmod_tmp.getTimeStr());
-      }
+      if (stream.fail ())
+        throw std::string ("There was a problem reading the timestamp from the undo.data file.");
 
       // 'time' is the first line of a modification
       // thus we will (re)set the taskmod object
-      tmod_tmp.reset(ts);
+      tmod_tmp.reset (ts);
 
-    } else if (line.substr (0, 3) == "old") {
-      tmod_tmp.setBefore(Task(line.substr(4)));
+    }
+    else if (line.substr (0, 3) == "old")
+    {
+      tmod_tmp.setBefore (Task (line.substr (4)));
 
-    } else if (line.substr (0, 3) == "new") {
-      tmod_tmp.setAfter(Task(line.substr(4)));
+    }
+    else if (line.substr (0, 3) == "new")
+    {
+      tmod_tmp.setAfter (Task (line.substr (4)));
 
       // 'new' is the last line of a modification,
       // thus we can push to the list
-      list.push_back(tmod_tmp);
+      list.push_back (tmod_tmp);
 
-      assert(tmod_tmp.isValid());
-      DEBUG_STR("  taskmod complete");
+      assert (tmod_tmp.isValid ());
+      DEBUG_STR ("  taskmod complete");
     }
   }
 
@@ -167,6 +172,7 @@ void TDB::clear ()
   mId = 1;
   mPending.clear ();
   mNew.clear ();
+  mCompleted.clear ();
   mModified.clear ();
 }
 
@@ -191,7 +197,8 @@ void TDB::lock (bool lockFile /* = true */)
 
     mPending.clear ();
     mNew.clear ();
-    mPending.clear ();
+    mCompleted.clear ();
+    mModified.clear ();
 
     foreach (location, mLocations)
     {
@@ -208,9 +215,10 @@ void TDB::lock (bool lockFile /* = true */)
 ////////////////////////////////////////////////////////////////////////////////
 void TDB::unlock ()
 {
-  mPending.clear ();
-  mNew.clear ();
-  mModified.clear ();
+  // Do not clear out these items, as they may be used in a read-only fashion.
+  // mPending.clear ();
+  // mNew.clear ();
+  // mModified.clear ();
 
   foreach (location, mLocations)
   {
@@ -304,6 +312,12 @@ int TDB::loadPending (std::vector <Task>& tasks, Filter& filter)
             task.id = mId++;
 
             mPending.push_back (task);
+
+            // Maintain mapping for ease of link/dependency resolution.
+            // Note that this mapping is not restricted by the filter, and is
+            // therefore a complete set.
+            mI2U[task.id] = task.get ("uuid");
+            mU2I[task.get ("uuid")] = task.id;
           }
 
           ++line_number;
@@ -420,11 +434,54 @@ int TDB::loadCompleted (std::vector <Task>& tasks, Filter& filter)
   return tasks.size ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::vector <Task>& TDB::getAllPending ()
+{
+  return mPending;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::vector <Task>& TDB::getAllNew ()
+{
+  return mNew;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::vector <Task>& TDB::getAllCompleted ()
+{
+  return mCompleted;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::vector <Task>& TDB::getAllModified ()
+{
+  return mModified;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Note: mLocations[0] is where all tasks are written.
 void TDB::add (const Task& task)
 {
-  mNew.push_back (task);
+  std::string unique;
+  Task t (task);
+  if (task.get ("uuid") == "")
+    unique = ::uuid ();
+  else
+    unique = task.get ("uuid");
+
+  t.set ("uuid", unique);
+
+  // If the tasks are loaded, then verify that this uuid is not already in
+  // the file.
+  if (uuidAlreadyUsed (unique, mNew)      ||
+      uuidAlreadyUsed (unique, mModified) ||
+      uuidAlreadyUsed (unique, mPending)  ||
+      uuidAlreadyUsed (unique, mCompleted))
+    throw std::string ("Cannot add task because the uuid '") + unique + "' is not unique.";
+
+  mNew.push_back (t);
+  mI2U[task.id] = unique;
+  mU2I[task.get ("uuid")] = t.id;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -534,6 +591,7 @@ int TDB::commit ()
 // Scans the pending tasks for any that are completed or deleted, and if so,
 // moves them to the completed.data file.  Returns a count of tasks moved.
 // Now reverts expired waiting tasks to pending.
+// Now cleans up dangling dependencies.
 int TDB::gc ()
 {
   Timer t ("TDB::gc");
@@ -554,6 +612,30 @@ int TDB::gc ()
   std::vector <Task> ignore;
   loadPending (ignore, filter);
 
+  // Search for dangling dependencies.  These are dependencies whose uuid cannot
+  // be converted to an id by TDB.
+  std::vector <std::string> deps;
+  foreach (task, mPending)
+  {
+    if (task->has ("depends"))
+    {
+      deps.clear ();
+      task->getDependencies (deps);
+      foreach (dep, deps)
+      {
+        if (id (*dep) == 0)
+        {
+          task->removeDependency (*dep);
+          ++count_pending_changes;
+          context.debug ("GC: Removed dangling dependency "
+                          + task->get ("uuid")
+                          + " -> "
+                          + *dep);
+        }
+      }
+    }
+  }
+
   // Now move completed and deleted tasks from the pending list to the
   // completed list.  Isn't garbage collection easy?
   std::vector <Task> still_pending;
@@ -577,6 +659,9 @@ int TDB::gc ()
         task->setStatus (Task::pending);
         task->remove ("wait");
         ++count_pending_changes;  // modification
+
+        context.debug (std::string ("TDB::gc waiting -> pending for ") +
+                       task->get ("uuid"));
       }
 
       still_pending.push_back (*task);
@@ -667,15 +752,18 @@ void TDB::undo ()
   Date lastChange (atoi (when.c_str ()));
 
   // Set the colors.
-  Color color_red   (context.config.get ("color.undo.before"));
-  Color color_green (context.config.get ("color.undo.after"));
+  bool useColor = context.config.getBoolean ("color") ||
+                  context.config.getBoolean ("_forcecolor") ? true : false;
+
+  Color color_red   (useColor ? context.config.get ("color.undo.before") : "");
+  Color color_green (useColor ? context.config.get ("color.undo.after") : "");
 
   if (context.config.get ("undo.style") == "side")
   {
-    std::cout << std::endl
+    std::cout << "\n"
               << "The last modification was made "
               << lastChange.toString ()
-              << std::endl;
+              << "\n";
 
     // Attributes are all there is, so figure the different attribute names
     // between before and after.
@@ -685,8 +773,16 @@ void TDB::undo ()
     table.addColumn (" ");
     table.addColumn ("Prior Values");
     table.addColumn ("Current Values");
-    table.setColumnUnderline (1);
-    table.setColumnUnderline (2);
+
+    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
+        context.config.getBoolean ("fontunderline"))
+    {
+      table.setColumnUnderline (1);
+      table.setColumnUnderline (2);
+    }
+    else
+      table.setTableDashedUnderline ();
+
     table.setColumnWidth (0, Table::minimum);
     table.setColumnWidth (1, Table::flexible);
     table.setColumnWidth (2, Table::flexible);
@@ -758,9 +854,9 @@ void TDB::undo ()
       }
     }
 
-    std::cout << std::endl
+    std::cout << "\n"
               << table.render ()
-              << std::endl;
+              << "\n";
   }
 
   // This style looks like this:
@@ -803,7 +899,7 @@ void TDB::undo ()
 
     row = table.addRow ();
     table.addCell (row, 0, "+++ current state ");  // Note trailing space.
-    table.addCell (row, 1, "Change made " + lastChange.toStringWithTime (context.config.get ("dateformat")));
+    table.addCell (row, 1, "Change made " + lastChange.toString (context.config.get ("dateformat")));
     table.setRowColor (row, color_green);
 
     table.addRow ();
@@ -891,14 +987,18 @@ void TDB::undo ()
       }
     }
 
-    std::cout << std::endl
+    std::cout << "\n"
               << table.render ()
-              << std::endl;
+              << "\n";
   }
 
   // Output displayed, now confirm.
   if (!confirm ("The undo command is not reversible.  Are you sure you want to revert to the previous state?"))
-    throw std::string ("No changes made.");
+  {
+    std::cout << "No changes made.\n";
+    context.hooks.trigger ("post-undo");
+    return;
+  }
 
   // Extract identifying uuid.
   std::string uuid;
@@ -923,12 +1023,12 @@ void TDB::undo ()
       if (prior != "")
       {
         *task = prior;
-        std::cout << "Modified task reverted." << std::endl;
+        std::cout << "Modified task reverted.\n";
       }
       else
       {
         p.erase (task);
-        std::cout << "Task removed." << std::endl;
+        std::cout << "Task removed.\n";
       }
 
       // Rewrite files.
@@ -960,7 +1060,7 @@ void TDB::undo ()
         File::write (completedFile, c);
         File::write (pendingFile, p);
         File::write (undoFile, u);
-        std::cout << "Modified task reverted." << std::endl;
+        std::cout << "Modified task reverted.\n";
         context.debug ("TDB::undo - task belongs in pending.data");
       }
       else
@@ -968,11 +1068,11 @@ void TDB::undo ()
         *task = prior;
         File::write (completedFile, c);
         File::write (undoFile, u);
-        std::cout << "Modified task reverted." << std::endl;
+        std::cout << "Modified task reverted.\n";
         context.debug ("TDB::undo - task belongs in completed.data");
       }
 
-      std::cout << "Undo complete." << std::endl;
+      std::cout << "Undo complete.\n";
       context.hooks.trigger ("post-undo");
       return;
     }
@@ -982,10 +1082,8 @@ void TDB::undo ()
   // Perhaps the task was in completed.data, which was still in file format 3?
   std::cout << "Task with UUID "
             << uuid.substr (6, 36)
-            << " not found in data."
-            << std::endl
-            << "No undo possible."
-            << std::endl;
+            << " not found in data.\n"
+            << "No undo possible.\n";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1005,9 +1103,8 @@ void TDB::merge (const std::string& mergeFile)
   // will contain the NEW undo.data
   std::vector <std::string> undo;
 
-  /////////////////////////////////////////////////////////////////////////////////////////
-  // initialize the files
-  /////////////////////////////////////////////////////////////////////////////////////////
+  ///////////////////////////////////////
+  // initialize the files:
 
   // load merge file (undo file of right/remote branch)
   std::vector <std::string> r;
@@ -1016,7 +1113,7 @@ void TDB::merge (const std::string& mergeFile)
 
   // file has to contain at least one entry
   if (r.size () < 3)
-    throw std::string ("There are no transactions to apply.");
+    throw std::string ("There are no changes to merge.");
 
   // load undo file (left/local branch)
   Directory location (context.config.get ("data.location"));
@@ -1030,179 +1127,221 @@ void TDB::merge (const std::string& mergeFile)
   std::vector <std::string>::iterator rit, lit;
 
   // read first line
-  rit = r.begin();
-  lit = l.begin();
+  rit = r.begin ();
+  lit = l.begin ();
 
-  rline = *rit;
-  lline = *lit;
+  if (rit != r.end())
+    rline = *rit;
+  if (lit != l.end())
+    lline = *lit;
 
-  /////////////////////////////////////////////////////////////////////////////////////////
-  // find the branch-off point
-  /////////////////////////////////////////////////////////////////////////////////////////
+  ///////////////////////////////////////
+  // find the branch-off point:
 
-  // first lines are not equal => assuming mergeFile starts at a later point in time
-  if (lline.compare(rline) != 0)
+  // first lines are not equal => assuming mergeFile starts at a
+  // later point in time
+  if (lline.compare (rline) != 0)
   {
     // iterate in local file to find rline
-    for (; lit != l.end(); ++lit)
+    for ( ; lit != l.end (); ++lit)
     {
       lline = *lit;
 
       // push the line to the new undo.data
-      undo.push_back(lline);
+      undo.push_back (lline + "\n");
 
       // found first matching lines?
-      if (lline.compare(rline) == 0)
+      if (lline.compare (rline) == 0)
         break;
     }
   }
 
+  // Add some color.
+  bool useColor = (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+                  ? true : false;
+  Color colorAdded    (context.config.get ("color.sync.added"));
+  Color colorChanged  (context.config.get ("color.sync.changed"));
+  Color colorRejected (context.config.get ("color.sync.rejected"));
+
   // at this point we can assume: (lline==rline) || (lit == l.end())
   // thus we search for the first non-equal lines or the EOF
   bool found = false;
-  for (; (lit != l.end()) && (rit != r.end()); ++lit, ++rit)
+  for ( ; (lit != l.end ()) && (rit != r.end ()); ++lit, ++rit)
   {
     lline = *lit;
     rline = *rit;
 
     // found first non-matching lines?
-    if (lline.compare(rline) != 0) {
+    if (lline.compare (rline) != 0)
+    {
       found = true;
       break;
-    } else {
+    }
+    else
+    {
       // push the line to the new undo.data
-      undo.push_back(lline + "\n");
+      undo.push_back (lline + "\n");
     }
   }
 
-  /////////////////////////////////////////////////////////////////////////////////////////
-  // branch-off point found
-  /////////////////////////////////////////////////////////////////////////////////////////
+  std::cout << "\n";
 
+  ///////////////////////////////////////
+  // branch-off point found:
   if (found)
   {
-    DEBUG_STR_PART("Branch-off point found at: ");
-    DEBUG_STR_END(lline);
+    DEBUG_STR_PART ("Branch-off point found at: ");
+    DEBUG_STR_END (lline);
 
     std::list<Taskmod> rmods;
 
     // helper lists
     std::set<std::string> uuid_new, uuid_left;
 
-    // ------ 1. read tasmods out of the remaining lines
-    readTaskmods(l, lit, lmods);
-    readTaskmods(r, rit, rmods);
+    // 1. read taskmods out of the remaining lines
+    readTaskmods (l, lit, lmods);
+    readTaskmods (r, rit, rmods);
 
-    // ------ 2. move new uuids into mods
-
-    DEBUG_STR_PART("adding new uuids (left) to skip list...");
+    // 2. move new uuids into mods
+    DEBUG_STR_PART ("adding new uuids (left) to skip list...");
 
     // modifications on the left side are already in the database
     // we just need them to merge conflicts, so we add new the mods for
     // new uuids to the skip-list 'uuid_left'
     std::list<Taskmod>::iterator lmod_it;
-    for (lmod_it = lmods.begin(); lmod_it != lmods.end(); lmod_it++) {
-      if ((*lmod_it).isNew()) {
-        std::cout << "Skipping the new local task        " << (*lmod_it).getUuid() << std::endl;
-        uuid_left.insert((*lmod_it).getUuid());
+    for (lmod_it = lmods.begin (); lmod_it != lmods.end (); lmod_it++)
+    {
+      if (lmod_it->isNew ())
+      {
+/*
+        std::cout << "New local task                     "
+                  << (useColor ? colorAdded.colorize (lmod_it->getUuid ()) : lmod_it->getUuid ())
+                  << "\n";
+*/
+
+        uuid_left.insert (lmod_it->getUuid ());
       }
     }
 
-    DEBUG_STR_END("done");
-    DEBUG_STR_PART("move new uuids (right) to redo list...");
+    DEBUG_STR_END ("done");
+    DEBUG_STR_PART ("move new uuids (right) to redo list...");
 
     // new items on the right side need to be inserted into the
     // local database
     std::list<Taskmod>::iterator rmod_it;
-    for (rmod_it = rmods.begin(); rmod_it != rmods.end();) {
+    for (rmod_it = rmods.begin (); rmod_it != rmods.end (); )
+    {
       // we have to save and increment the iterator because we may want to delete
       // the object from the list
       std::list<Taskmod>::iterator current = rmod_it++;
       Taskmod tmod = *current;
 
       // new uuid?
-      if (tmod.isNew()) {
-        std::cout << "Adding the new remote task         " << tmod.getUuid() << std::endl;
-        uuid_new.insert(tmod.getUuid());
-        mods.push_back(tmod);
-        rmods.erase(current);
+      if (tmod.isNew ())
+      {
+/*
+        std::cout << "Adding new remote task             "
+                  << (useColor ? colorAdded.colorize (tmod.getUuid ()) : tmod.getUuid ())
+                  << "\n";
+*/
+
+        uuid_new.insert (tmod.getUuid ());
+        mods.push_back (tmod);
+        rmods.erase (current);
       }
-      else if (uuid_new.find(tmod.getUuid()) != uuid_new.end()) { // uuid of modification was new
-        mods.push_back(tmod);
-        rmods.erase(current);
+      else if (uuid_new.find (tmod.getUuid ()) != uuid_new.end ())
+      {
+        // uuid of modification was new
+        mods.push_back (tmod);
+        rmods.erase (current);
       }
     }
 
-    DEBUG_STR_END("done");
+    DEBUG_STR_END ("done");
 
-    DEBUG_STR("Merging modifications:");
+    ///////////////////////////////////////
+    // merge modifications:
+    DEBUG_STR ("Merging modifications:");
 
-    // ------ 3. merge modifications
     // we iterate backwards to resolve conflicts by timestamps (newest one wins)
     std::list<Taskmod>::reverse_iterator lmod_rit;
     std::list<Taskmod>::reverse_iterator rmod_rit;
-    for (lmod_rit = lmods.rbegin(); lmod_rit != lmods.rend(); ++lmod_rit) {
-      Taskmod tmod_l = *lmod_rit;
-      std::string uuid = tmod_l.getUuid();
+    for (lmod_rit = lmods.rbegin (); lmod_rit != lmods.rend (); ++lmod_rit)
+    {
+      Taskmod     tmod_l = *lmod_rit;
+      std::string uuid   = tmod_l.getUuid ();
 
-      DEBUG_STR("  left uuid: " + uuid);
+      DEBUG_STR ("  left uuid: " + uuid);
 
       // skip if uuid had already been merged
-      if (uuid_left.find(uuid) == uuid_left.end()) {
-
+      if (uuid_left.find (uuid) == uuid_left.end ())
+      {
         bool rwin = false;
         bool lwin = false;
-        for (rmod_rit = rmods.rbegin(); rmod_rit != rmods.rend(); rmod_rit++) {
+        for (rmod_rit = rmods.rbegin (); rmod_rit != rmods.rend (); rmod_rit++)
+        {
           Taskmod tmod_r = *rmod_rit;
 
-          DEBUG_STR("    right uuid: " + tmod_r.getUuid());
-          if (tmod_r.getUuid() == uuid) {
-            DEBUG_STR("    uuid match found for " + uuid);
+          DEBUG_STR ("    right uuid: " + tmod_r.getUuid ());
+          if (tmod_r.getUuid () == uuid)
+          {
+            DEBUG_STR ("    uuid match found for " + uuid);
 
             // we already decided to take the mods from the right side
             // but we have to find the first modification newer than
             // the one on the left side to merge the history too
-            if (rwin) {
-              DEBUG_STR("    scanning right side");
-              if (tmod_r > tmod_l) {
-                mods.push_front(tmod_r);
-              }
+            if (rwin)
+            {
+              DEBUG_STR ("    scanning right side");
+              if (tmod_r > tmod_l)
+                mods.push_front (tmod_r);
 
-              std::list<Taskmod>::iterator tmp_it = rmod_rit.base();
-              rmods.erase(--tmp_it);
+              std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
+              rmods.erase (--tmp_it);
               rmod_rit--;
             }
-            else if (lwin) {
-              DEBUG_STR("    cleaning up right side");
+            else if (lwin)
+            {
+              DEBUG_STR ("    cleaning up right side");
 
-              std::list<Taskmod>::iterator tmp_it = rmod_rit.base();
-              rmods.erase(--tmp_it);
+              std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
+              rmods.erase (--tmp_it);
               rmod_rit--;
             }
-            else {
+            else
+            {
               // which one is newer?
-              if (tmod_r > tmod_l) {
-                std::cout << "Applying remote changes for uuid   " << uuid << std::endl;
+              if (tmod_r > tmod_l)
+              {
+                std::cout << "Found remote change to        "
+                          << (useColor ? colorChanged.colorize (uuid) : uuid)
+                          << "  \"" << cutOff (tmod_r.getBefore ().get ("description"), 10) << "\""
+                          << "\n";
 
                 mods.push_front(tmod_r);
 
                 // delete tmod from right side
-                std::list<Taskmod>::iterator tmp_it = rmod_rit.base();
-                rmods.erase(--tmp_it);
+                std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
+                rmods.erase (--tmp_it);
                 rmod_rit--;
 
                 rwin = true;
               }
-              else {
-                std::cout << "Rejecting remote changes for uuid  " << uuid << std::endl;
+              else
+              {
+                std::cout << "Retaining local changes to    "
+                          << (useColor ? colorRejected.colorize (uuid) : uuid)
+                          << "  \"" << cutOff (tmod_l.getBefore ().get ("description"), 10) << "\""
+                          << "\n";
+
                 // inserting right mod into history of local database
                 // so that it can be restored later
 
                 // TODO feature: make rejected changes on the remote branch restorable
 //                Taskmod reverse_tmod;
 //
-//                tmod_r.setBefore((*lmod_rit).getAfter());
-//                tmod_r.setTimestamp((*lmod_rit).getTimestamp()+1);
+//                tmod_r.setBefore(lmod_rit->getAfter());
+//                tmod_r.setTimestamp(lmod_rit->getTimestamp()+1);
 //
 //                reverse_tmod.setAfter(tmod_r.getBefore());
 //                reverse_tmod.setBefore(tmod_r.getAfter());
@@ -1212,57 +1351,71 @@ void TDB::merge (const std::string& mergeFile)
 //                mods.push_back(reverse_tmod);
 
                 // delete tmod from right side
-                std::list<Taskmod>::iterator tmp_it = rmod_rit.base();
-                rmods.erase(--tmp_it);
+                std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
+                rmods.erase (--tmp_it);
                 rmod_rit--;
 
                 // mark this uuid as merged
-                uuid_left.insert(uuid);
+                uuid_left.insert (uuid);
                 lwin = true;
               }
             }
           }
         } // for
 
-        if (rwin) {
-          DEBUG_STR("  concat the first match to left branch");
+        if (rwin)
+        {
+          DEBUG_STR ("  concat the first match to left branch");
           // concat the oldest (but still newer) modification on the right
           // to the endpoint on the left
-          mods.front().setBefore(tmod_l.getAfter());
+          mods.front ().setBefore(tmod_l.getAfter ());
         }
       }
     } // for
 
-    DEBUG_STR("adding non-conflicting changes from the right branch");
-    mods.splice(mods.begin(), rmods);
-
-    DEBUG_STR("sorting taskmod list");
-    mods.sort();
+    DEBUG_STR ("adding non-conflicting changes from the right branch");
+    mods.splice (mods.begin (), rmods);
 
+    DEBUG_STR ("sorting taskmod list");
+    mods.sort ();
   }
-  else if (rit == r.end())
+  else if (rit == r.end ())
   {
     // nothing happend on the remote branch
     // local branch is up-to-date
-    throw std::string ("Database is already up-to-date.");
+
+    // nothing happend on the local branch either
+
+    // break, to suppress autopush
+    if (lit == l.end ())
+    {
+      mods.clear ();
+      lmods.clear ();
+      throw std::string ("Database is up-to-date, no merge required.");
+    }
+
   }
-  else // lit == undo.end()
+  else // lit == l.end ()
   {
     // nothing happend on the local branch
-    std::cout << "No changes were made on the local database. Appending changes..." << std::endl;
+/*
+    std::cout << "No local changes detected.\n";
+*/
 
     // add remaining lines (remote branch) to the list of modifications
-    readTaskmods(r, rit, mods);
+/*
+    std::cout << "Remote changes detected.\n";
+*/
+    readTaskmods (r, rit, mods);
   }
 
-  ////////////////////////////////////////////////////////////
-  // redo command
-  ////////////////////////////////////////////////////////////
+  ///////////////////////////////////////
+  // Now apply the changes.
+  // redo command:
 
-  if (!mods.empty()) {
-    std::cout << "Running redo routine..." << std::endl;
-
-    std::string pendingFile = location.data + "/pending.data";
+  if (!mods.empty ())
+  {
+    std::string pendingFile   = location.data + "/pending.data";
     std::vector <std::string> pending;
 
     std::string completedFile = location.data + "/completed.data";
@@ -1271,52 +1424,61 @@ void TDB::merge (const std::string& mergeFile)
     if (! File::read (pendingFile, pending))
       throw std::string ("Could not read '") + pendingFile + "'.";
 
-    if (! File::read(completedFile, completed))
+    if (! File::read (completedFile, completed))
       throw std::string ("Could not read '") + completedFile + "'.";
 
     // iterate over taskmod list
     std::list<Taskmod>::iterator it;
-    for (it = mods.begin(); it != mods.end();) {
+    for (it = mods.begin (); it != mods.end (); )
+    {
       std::list<Taskmod>::iterator current = it++;
       Taskmod tmod = *current;
 
       // Modification to an existing task.
-      if (!tmod.isNew()) {
-        std::string uuid = tmod.getUuid();
-        Task::status statusBefore = tmod.getBefore().getStatus();
-        Task::status statusAfter  = tmod.getAfter().getStatus();
+      if (!tmod.isNew ())
+      {
+        std::string uuid = tmod.getUuid ();
+        Task::status statusBefore = tmod.getBefore().getStatus ();
+        Task::status statusAfter  = tmod.getAfter().getStatus ();
 
         std::vector <std::string>::iterator it;
 
         bool found = false;
         if ( (statusBefore == Task::completed)
-          || (statusBefore == Task::deleted) ) {
-
+          || (statusBefore == Task::deleted) )
+        {
           // Find the same uuid in completed data
           for (it = completed.begin (); it != completed.end (); ++it)
           {
             if (it->find (uuid) != std::string::npos)
             {
               // Update the completed record.
-              std::cout << "Modifying            " << uuid << std::endl;
+/*
+              std::cout << "Modifying                     "
+                        << (useColor ? colorChanged.colorize (uuid) : uuid)
+                        << "\n";
+*/
 
               // remove the \n from composeF4() string
-              std::string newline = tmod.getAfter().composeF4();
-              newline = newline.substr(0, newline.length()-1);
+              std::string newline = tmod.getAfter ().composeF4 ();
+              newline = newline.substr (0, newline.length ()-1);
 
               // does the tasks move to pending data?
               // this taskmod will not arise from
               // normal usage of task, but those kinds of
               // taskmods may be constructed to merge databases
               if ( (statusAfter != Task::completed)
-                && (statusAfter != Task::deleted) ) {
+                && (statusAfter != Task::deleted) )
+              {
                 // insert task into pending data
-                pending.push_back(newline);
+                pending.push_back (newline);
 
                 // remove task from completed data
-                completed.erase(it);
+                completed.erase (it);
 
-              } else {
+              }
+              else
+              {
                 // replace the current line
                 *it = newline;
               }
@@ -1325,31 +1487,38 @@ void TDB::merge (const std::string& mergeFile)
               break;
             }
           }
-        } else {
-
+        }
+        else
+        {
           // Find the same uuid in the pending data.
           for (it = pending.begin (); it != pending.end (); ++it)
           {
             if (it->find (uuid) != std::string::npos)
             {
               // Update the pending record.
-              std::cout << "Modifying            " << uuid << std::endl;
+              std::cout << "Found remote change to        "
+                        << (useColor ? colorChanged.colorize (uuid) : uuid)
+                        << "  \"" << cutOff (tmod.getBefore ().get ("description"), 10) << "\""
+                        << "\n";
 
               // remove the \n from composeF4() string
               // which will replace the current line
-              std::string newline = tmod.getAfter().composeF4();
-              newline = newline.substr(0, newline.length()-1);
+              std::string newline = tmod.getAfter ().composeF4 ();
+              newline = newline.substr (0, newline.length ()-1);
 
               // does the tasks move to completed data
               if ( (statusAfter == Task::completed)
-                || (statusAfter == Task::deleted) ) {
+                || (statusAfter == Task::deleted) )
+              {
                 // insert task into completed data
-                completed.push_back(newline);
+                completed.push_back (newline);
 
                 // remove task from pending data
-                pending.erase(it);
+                pending.erase (it);
 
-              } else {
+              }
+              else
+              {
                 // replace the current line
                 *it = newline;
               }
@@ -1360,15 +1529,19 @@ void TDB::merge (const std::string& mergeFile)
           }
         }
 
-        if (!found) {
-          std::cout << "Missing              " << uuid << std::endl;
-          mods.erase(current);
+        if (!found)
+        {
+          std::cout << "Missing                       "
+                    << (useColor ? colorRejected.colorize (uuid) : uuid)
+                    << "  \"" << cutOff (tmod.getBefore ().get ("description"), 10) << "\""
+                    << "\n";
+          mods.erase (current);
         }
-
       }
-      else {
+      else
+      {
         // Check for dups.
-        std::string uuid = tmod.getAfter().get("uuid");
+        std::string uuid = tmod.getAfter ().get ("uuid");
 
         // Find the same uuid in the pending data.
         bool found = false;
@@ -1384,12 +1557,20 @@ void TDB::merge (const std::string& mergeFile)
 
         if (!found)
         {
-          std::cout << "Adding               " << uuid << std::endl;
-          pending.push_back (tmod.getAfter().composeF4());
+          std::cout << "Merging new remote task       "
+                    << (useColor ? colorAdded.colorize (uuid) : uuid)
+                    << "  \"" << cutOff (tmod.getAfter ().get ("description"), 10) << "\""
+                    << "\n";
+
+          // remove the \n from composeF4() string
+          std::string newline = tmod.getAfter ().composeF4 ();
+          newline = newline.substr (0, newline.length ()-1);
+          pending.push_back (newline);
         }
         else
-          std::cout << "Not adding duplicate " << uuid << std::endl;
-          mods.erase(current);
+        {
+          mods.erase (current);
+        }
       }
     }
 
@@ -1404,25 +1585,40 @@ void TDB::merge (const std::string& mergeFile)
     // at this point undo contains the lines up to the branch-off point
     // now we merge mods (new modifications from mergefile)
     // with lmods (part of old undo.data)
-    mods.merge(lmods);
+    mods.merge (lmods);
 
     // generate undo.data format
-    for (it = mods.begin(); it != mods.end(); it++) {
-      undo.push_back(it->toString());
-    }
+    for (it = mods.begin (); it != mods.end (); it++)
+      undo.push_back(it->toString ());
 
     // write undo file
     if (! File::write (undoFile, undo, false))
       throw std::string ("Could not write '") + undoFile + "'.";
   }
-  else // nothing to be done
-  {
-    std::cout << "nothing to be done" << std::endl;
-  }
 
   // delete objects
-  lmods.clear();
-  mods.clear();
+  lmods.clear ();
+  mods.clear ();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+std::string TDB::uuid (int id) const
+{
+  std::map <int, std::string>::const_iterator i;
+  if ((i = mI2U.find (id)) != mI2U.end ())
+    return i->second;
+
+  return "";
+}
+
+////////////////////////////////////////////////////////////////////////////////
+int TDB::id (const std::string& uuid) const
+{
+  std::map <std::string, int>::const_iterator i;
+  if ((i = mU2I.find (uuid)) != mU2I.end ())
+    return i->second;
+
+  return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1448,8 +1644,8 @@ FILE* TDB::openAndLock (const std::string& file)
   if (mLock)
     while (flock (fileno (in), LOCK_NB | LOCK_EX) && ++retry <= 3)
     {
-      std::cout << "Waiting for file lock..." << std::endl;
-      while (flock (fileno (in), LOCK_EX) && ++retry <= 3)
+      std::cout << "Waiting for file lock...\n";
+      while (flock (fileno (in), LOCK_NB | LOCK_EX) && ++retry <= 3)
         delay (0.2);
     }
 
@@ -1479,3 +1675,16 @@ void TDB::writeUndo (const Task& before, const Task& after, FILE* file)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+bool TDB::uuidAlreadyUsed (
+  const std::string& uuid,
+  const std::vector <Task>& all)
+{
+  std::vector <Task>::const_iterator it;
+  for (it = all.begin (); it != all.end (); ++it)
+    if (it->get ("uuid") == uuid)
+      return true;
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
