@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -34,6 +34,7 @@
 #include "Task.h"
 #include "text.h"
 #include "util.h"
+#include "main.h"
 
 extern Context context;
 
@@ -163,7 +164,7 @@ void Task::legacyParse (const std::string& line)
   {
   // File format version 1, from 2006.11.27 - 2007.12.31
   case 1:
-    throw std::string ("Task no longer supports file format 1, originally used "
+    throw std::string ("Taskwarrior no longer supports file format 1, originally used "
                        "between 27 November 2006 and 31 December 2007.");  // TODO i18n
     break;
 
@@ -327,7 +328,7 @@ void Task::legacyParse (const std::string& line)
     break;
 
   default:
-    throw std::string ("Unrecognized task file format."); // TODO i18n
+    throw std::string ("Unrecognized taskwarrior file format."); // TODO i18n
     break;
   }
 }
@@ -392,6 +393,36 @@ std::string Task::composeCSV () const
   return out.str ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+std::string Task::composeYAML () const
+{
+  std::stringstream out;
+
+  // Task header.
+  out << "  task:\n";
+
+  // Get all the supported attribute names.
+  std::vector <std::string> names;
+  Att::allNames (names);
+  std::sort (names.begin (), names.end ());
+
+  // Only include non-trivial attributes.
+  std::string value;
+  foreach (name, names)
+    if ((value = get (*name)) != "")
+      out << "    " << *name << ": " << value << "\n";
+
+  // Now the annotations, which are not listed by the Att::allNames call.
+  std::vector <Att> annotations;
+  getAnnotations (annotations);
+  foreach (a, annotations)
+    out << "    annotation:\n"
+        << "      entry: "       << a->name().substr (11) << "\n"
+        << "      description: " << a->value ()           << "\n";
+
+  return out.str ();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void Task::getAnnotations (std::vector <Att>& annotations) const
 {
@@ -440,6 +471,93 @@ void Task::removeAnnotations ()
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+void Task::addDependency (int id)
+{
+  if (id == this->id)
+    throw std::string ("A task cannot be dependent on itself.");
+
+  // Check for extant dependency.
+  std::string uuid = context.tdb.uuid (id);
+  if (uuid == "")
+  {
+    std::stringstream s;
+    s << "Could not create a dependency on task " << id << " - not found.";
+    throw s.str ();
+  }
+
+  // Store the dependency.
+  std::string depends = get ("depends");
+  if (depends.length ())
+  {
+    if (depends.find (uuid) == std::string::npos)
+      set ("depends", depends + "," + uuid);
+    else
+    {
+      std::stringstream out;
+      out << "Task " << this->id << " already depends on task " << id << ".";
+      throw out.str ();
+    }
+  }
+  else
+    set ("depends", uuid);
+
+  // Prevent circular dependencies.
+  if (dependencyIsCircular (*this))
+    throw std::string ("Circular dependency detected and disallowed.");
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void Task::removeDependency (const std::string& uuid)
+{
+  std::vector <std::string> deps;
+  split (deps, get ("depends"), ',');
+
+  std::vector <std::string>::iterator i;
+  i = std::find (deps.begin (), deps.end (), uuid);
+  if (i != deps.end ())
+  {
+    deps.erase (i);
+    std::string combined;
+    join (combined, ",", deps);
+    set ("depends", combined);
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void Task::removeDependency (int id)
+{
+  std::string uuid = context.tdb.uuid (id);
+  if (uuid != "")
+    removeDependency (uuid);
+  else
+  {
+    std::stringstream s;
+    s << "Could not find a UUID for id " << id << ".";
+    throw s.str ();
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void Task::getDependencies (std::vector <int>& all) const
+{
+  std::vector <std::string> deps;
+  split (deps, get ("depends"), ',');
+
+  all.clear ();
+
+  std::vector <std::string>::iterator i;
+  for (i = deps.begin (); i != deps.end (); ++i)
+    all.push_back (context.tdb.id (*i));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void Task::getDependencies (std::vector <std::string>& all) const
+{
+  all.clear ();
+  split (all, get ("depends"), ',');
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int Task::getTagCount ()
 {
@@ -516,21 +634,21 @@ void Task::validate () const
   if (!has ("uuid")  ||
       !has ("entry") ||
       !has ("description"))
-    throw std::string ("A task must have a description in order to be valid."); // TODO i18n
+    throw std::string ("A task must have a description in order to be valid.");
 
   if (get ("description") == "") // No i18n
-    throw std::string ("Cannot add a task that is blank, or contains <CR> or <LF> characters."); // TODO i18n
+    throw std::string ("Cannot add a task that is blank, or contains <CR> or <LF> characters.");
 
   if (has ("due"))
   {
     Date due (::atoi (get ("due").c_str ()));
 
-    // Verify until > due
-    if (has ("until"))
+    // Verify wait < due
+    if (has ("wait"))
     {
-      Date until (::atoi (get ("until").c_str ()));
-      if (due > until)
-        throw std::string ("An 'until' date must be after a 'due' date."); // TODO i18n
+      Date wait (::atoi (get ("wait").c_str ()));
+      if (wait > due)
+        throw std::string ("A 'wait' date must be before a 'due' date.");
     }
 
     Date entry (::atoi (get ("entry").c_str ()));
@@ -539,24 +657,36 @@ void Task::validate () const
     {
       Date start (::atoi (get ("start").c_str ()));
       if (entry > start)
-        throw std::string ("A 'start' date must be after an 'entry' date."); // TODO i18n
+        throw std::string ("A 'start' date must be after an 'entry' date.");
     }
 
     if (has ("end"))
     {
       Date end (::atoi (get ("end").c_str ()));
       if (entry > end)
-        throw std::string ("An 'end' date must be after an 'entry' date."); // TODO i18n
+        throw std::string ("An 'end' date must be after an 'entry' date.");
     }
   }
+  else
+  {
+    if (has ("recur"))
+      throw std::string ("You cannot specify a recurring task without a due date.");
+  }
+
+  if (has ("until") && !has ("recur"))
+    throw std::string ("You cannot specify an until date for a non-recurring task.");
 
   // Recur durations must be valid.
   if (has ("recur"))
   {
     Duration d;
     if (! d.valid (get ("recur")))
-      throw std::string ("A recurrence value must be valid."); // TODO i18n
+      throw std::string ("A recurrence value must be valid.");
   }
+
+  if (has ("wait") &&
+      getStatus () == Task::recurring)
+    throw std::string ("You cannot create a task that is both waiting and recurring.");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -633,3 +763,205 @@ int Task::determineVersion (const std::string& line)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// Urgency is defined as a polynomial, the value of which is calculated in this
+// function, according to:
+//
+//   U = A.t  + B.t  + C.t  ...
+//          a      b      c
+//
+//   U       = urgency
+//   A       = coefficient for term a
+//   t sub a = numeric scale from 0 -> 1, with 1 being the highest
+//             urgency, derived from one task attribute and mapped
+//             to the numeric scale
+//
+// See rfc31-urgency.txt for full details.
+//
+double Task::urgency ()
+{
+  double urgency = 0.0;
+
+  // urgency.priority.coefficient
+  double coefficient = context.config.getReal ("urgency.priority.coefficient");
+  double term;
+
+  std::string value = get ("priority");
+       if (value == "H") term = 1.0;
+  else if (value == "M") term = 0.65;
+  else if (value == "L") term = 0.3;
+  else                   term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.project.coefficient
+  coefficient = context.config.getReal ("urgency.project.coefficient");
+
+  value = get ("project");
+  if (value != "") term = 1.0;
+  else             term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.active.coefficient
+  coefficient = context.config.getReal ("urgency.active.coefficient");
+
+  value = get ("start");
+  if (value != "") term = 1.0;
+  else             term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.waiting.coefficient
+  coefficient = context.config.getReal ("urgency.waiting.coefficient");
+
+  value = get ("status");
+       if (value == "pending") term = 1.0;
+  else if (value == "waiting") term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.blocked.coefficient
+  coefficient = context.config.getReal ("urgency.blocked.coefficient");
+
+  value = get ("depends");
+  if (value != "") term = 0.0;
+  else             term = 1.0;
+
+  urgency += term * coefficient;
+
+  // urgency.annotations.coefficient
+  coefficient = context.config.getReal ("urgency.annotations.coefficient");
+
+  std::vector <Att> annos;
+  getAnnotations (annos);
+       if (annos.size () >= 3) term = 1.0;
+  else if (annos.size () == 2) term = 0.9;
+  else if (annos.size () == 1) term = 0.8;
+  else                         term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.tags.coefficient
+  coefficient = context.config.getReal ("urgency.tags.coefficient");
+
+  int count = getTagCount ();
+       if (count >= 3) term = 1.0;
+  else if (count == 2) term = 0.9;
+  else if (count == 1) term = 0.8;
+  else                 term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.next.coefficient
+  coefficient = context.config.getReal ("urgency.next.coefficient");
+
+  if (hasTag ("next")) term = 1.0;
+  else                 term = 0.0;
+
+  urgency += term * coefficient;
+
+  // urgency.due.coefficient
+  // overdue days 7 -> 1.0
+  //              6 -> 0.96
+  //              5 -> 0.92
+  //              5 -> 0.88
+  //              5 -> 0.84
+  //              5 -> 0.80
+  //              5 -> 0.76
+  //              0 -> 0.72
+  //             -1 -> 0.68
+  //             -2 -> 0.64
+  //             -3 -> 0.60
+  //             -4 -> 0.56
+  //             -5 -> 0.52
+  //             -6 -> 0.48
+  //             -7 -> 0.44
+  //             -8 -> 0.40
+  //             -9 -> 0.36
+  //            -10 -> 0.32
+  //            -11 -> 0.28
+  //            -12 -> 0.24
+  //            -13 -> 0.20
+  //            -14 -> 0.16
+  //    no due date -> 0.0
+  coefficient = context.config.getReal ("urgency.due.coefficient");
+  if (has ("due"))
+  {
+    Date now;
+    Date due (get ("due"));
+    int days_overdue = (now - due) / 86400;
+
+         if (days_overdue >=  7)  term = 1.0;
+    else if (days_overdue >=  6)  term = 0.96;
+    else if (days_overdue >=  5)  term = 0.92;
+    else if (days_overdue >=  4)  term = 0.88;
+    else if (days_overdue >=  3)  term = 0.84;
+    else if (days_overdue >=  2)  term = 0.80;
+    else if (days_overdue >=  1)  term = 0.76;
+    else if (days_overdue >=  0)  term = 0.72;
+    else if (days_overdue >= -1)  term = 0.68;
+    else if (days_overdue >= -2)  term = 0.64;
+    else if (days_overdue >= -3)  term = 0.60;
+    else if (days_overdue >= -4)  term = 0.56;
+    else if (days_overdue >= -5)  term = 0.52;
+    else if (days_overdue >= -6)  term = 0.48;
+    else if (days_overdue >= -7)  term = 0.44;
+    else if (days_overdue >= -8)  term = 0.40;
+    else if (days_overdue >= -9)  term = 0.36;
+    else if (days_overdue >= -10) term = 0.32;
+    else if (days_overdue >= -11) term = 0.28;
+    else if (days_overdue >= -12) term = 0.24;
+    else if (days_overdue >= -13) term = 0.20;
+    else                          term = 0.16;
+  }
+  else
+    term = 0.0;
+
+  urgency += term * coefficient;
+
+  // Tag- and project-specific coefficients.
+  std::vector <std::string> all;
+  context.config.all (all);
+
+  foreach (var, all)
+  {
+    if (var->substr (0, 13) == "urgency.user.")
+    {
+      // urgency.user.project.<project>.coefficient
+      std::string::size_type end = std::string::npos;
+      if (var->substr (13, 8) == "project." &&
+          (end = var->find (".coefficient")) != std::string::npos)
+      {
+        std::string project = var->substr (21, end - 21);
+        coefficient = context.config.getReal (*var);
+
+        if (get ("project").find (project) == 0)
+          urgency += coefficient;
+      }
+
+    // urgency.user.tag.<tag>.coefficient
+      if (var->substr (13, 4) == "tag." &&
+          (end = var->find (".coefficient")) != std::string::npos)
+      {
+        std::string tag = var->substr (17, end - 17);
+        coefficient = context.config.getReal (*var);
+
+        if (hasTag (tag))
+          urgency += coefficient;
+      }
+    }
+  }
+
+  // urgency.blocking.coefficient
+  coefficient = context.config.getReal ("urgency.blocking.coefficient");
+
+  if (dependencyIsBlocking (*this)) term = 1.0;
+  else                              term = 0.0;
+
+  urgency += term * coefficient;
+
+  // Return the sum of all terms.
+  return urgency;
+}
+
+////////////////////////////////////////////////////////////////////////////////
