@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -53,63 +53,7 @@ static std::vector <std::string> customReports;
 ////////////////////////////////////////////////////////////////////////////////
 // This report will eventually become the one report that many others morph into
 // via the .taskrc file.
-int handleCustomReport (const std::string& report, std::string &outs)
-{
-  // Load report configuration.
-  std::string columnList = context.config.get ("report." + report + ".columns");
-  std::string labelList  = context.config.get ("report." + report + ".labels");
-  std::string sortList   = context.config.get ("report." + report + ".sort");
-  std::string filterList = context.config.get ("report." + report + ".filter");
-
-  std::vector <std::string> filterArgs;
-  split (filterArgs, filterList, ' ');
-  {
-    Cmd cmd (report);
-    Task task;
-    Sequence sequence;
-    Subst subst;
-    Filter filter;
-    context.parse (filterArgs, cmd, task, sequence, subst, filter);
-
-    context.sequence.combine (sequence);
-
-    // Allow limit to be overridden by the command line.
-    if (!context.task.has ("limit") && task.has ("limit"))
-      context.task.set ("limit", task.get ("limit"));
-
-    foreach (att, filter)
-      context.filter.push_back (*att);
-  }
-
-  // Get all the tasks.
-  std::vector <Task> tasks;
-  context.tdb.lock (context.config.getBoolean ("locking"));
-  handleRecurrence ();
-  context.tdb.load (tasks, context.filter);
-  context.tdb.commit ();
-  context.tdb.unlock ();
-
-  return runCustomReport (
-    report,
-    columnList,
-    labelList,
-    sortList,
-    filterList,
-    tasks,
-    outs);
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// This report will eventually become the one report that many others morph into
-// via the .taskrc file.
-int runCustomReport (
-  const std::string& report,
-  const std::string& columnList,
-  const std::string& labelList,
-  const std::string& sortList,
-  const std::string& filterList,
-  std::vector <Task>& tasks,
-  std::string &outs)
+int handleCustomReport (const std::string& report, std::string& outs)
 {
   int rc = 0;
 
@@ -117,12 +61,17 @@ int runCustomReport (
       context.hooks.trigger (std::string ("pre-") + report + "-command"))
   {
     // Load report configuration.
+    std::string reportColumns = context.config.get ("report." + report + ".columns");
+    std::string reportLabels  = context.config.get ("report." + report + ".labels");
+    std::string reportSort    = context.config.get ("report." + report + ".sort");
+    std::string reportFilter  = context.config.get ("report." + report + ".filter");
+
     std::vector <std::string> columns;
-    split (columns, columnList, ',');
+    split (columns, reportColumns, ',');
     validReportColumns (columns);
 
     std::vector <std::string> labels;
-    split (labels, labelList, ',');
+    split (labels, reportLabels, ',');
 
     if (columns.size () != labels.size () && labels.size () != 0)
       throw std::string ("There are a different number of columns than labels ") +
@@ -134,22 +83,26 @@ int runCustomReport (
         columnLabels[columns[i]] = labels[i];
 
     std::vector <std::string> sortOrder;
-    split (sortOrder, sortList, ',');
+    split (sortOrder, reportSort, ',');
     validSortColumns (columns, sortOrder);
 
+    // Apply rc overrides.
     std::vector <std::string> filterArgs;
-    split (filterArgs, filterList, ' ');
+    std::vector <std::string> filteredArgs;
+    split (filterArgs, reportFilter, ' ');
+    context.applyOverrides (filterArgs, filteredArgs);
+
     {
       Cmd cmd (report);
       Task task;
       Sequence sequence;
       Subst subst;
       Filter filter;
-      context.parse (filterArgs, cmd, task, sequence, subst, filter);
+      context.parse (filteredArgs, cmd, task, sequence, subst, filter);
 
       context.sequence.combine (sequence);
 
-      // Allow limit to be overridden by the command line.
+      // Special case: Allow limit to be overridden by the command line.
       if (!context.task.has ("limit") && task.has ("limit"))
         context.task.set ("limit", task.get ("limit"));
 
@@ -157,13 +110,18 @@ int runCustomReport (
         context.filter.push_back (*att);
     }
 
+    // Get all the tasks.
+    std::vector <Task> tasks;
+    context.tdb.lock (context.config.getBoolean ("locking"));
+    handleRecurrence ();
+    context.tdb.load (tasks, context.filter);
+    context.tdb.commit ();
+    context.tdb.unlock ();
+
     // Filter sequence.
     if (context.sequence.size ())
       context.filter.applySequence (tasks, context.sequence);
 
-    // Initialize colorization for subsequent auto colorization.
-    initializeColorRules ();
-
     Table table;
     table.setTableWidth (context.getWidth ());
     table.setDateFormat (context.config.get ("dateformat"));
@@ -274,7 +232,7 @@ int runCustomReport (
         }
       }
 
-      else if (*col == "entry")
+      else if (*col == "entry" || *col == "entry_time")
       {
         table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Added");
         table.setColumnWidth (columnCount, Table::minimum);
@@ -294,27 +252,7 @@ int runCustomReport (
         }
       }
 
-      else if (*col == "entry_time")
-      {
-        table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Added");
-        table.setColumnWidth (columnCount, Table::minimum);
-        table.setColumnJustification (columnCount, Table::right);
-
-        std::string entered;
-        for (unsigned int row = 0; row < tasks.size(); ++row)
-        {
-          entered = tasks[row].get ("entry");
-          if (entered.length ())
-          {
-            Date dt (::atoi (entered.c_str ()));
-            entered = dt.toStringWithTime (context.config.get ("dateformat"));
-            context.hooks.trigger ("format-entry_time", "entry_time", entered);
-            table.addCell (row, columnCount, entered);
-          }
-        }
-      }
-
-      else if (*col == "start")
+      else if (*col == "start" || *col == "start_time")
       {
         table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Started");
         table.setColumnWidth (columnCount, Table::minimum);
@@ -334,27 +272,7 @@ int runCustomReport (
         }
       }
 
-      else if (*col == "start_time")
-      {
-        table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Started");
-        table.setColumnWidth (columnCount, Table::minimum);
-        table.setColumnJustification (columnCount, Table::right);
-
-        std::string started;
-        for (unsigned int row = 0; row < tasks.size(); ++row)
-        {
-          started = tasks[row].get ("start");
-          if (started.length ())
-          {
-            Date dt (::atoi (started.c_str ()));
-            started = dt.toStringWithTime (context.config.get ("dateformat"));
-            context.hooks.trigger ("format-start_time", "start_time", started);
-            table.addCell (row, columnCount, started);
-          }
-        }
-      }
-
-      else if (*col == "end")
+      else if (*col == "end" || *col == "end_time")
       {
         table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Completed");
         table.setColumnWidth (columnCount, Table::minimum);
@@ -374,28 +292,6 @@ int runCustomReport (
         }
       }
 
-      else if (*col == "end_time")
-      {
-        table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Completed");
-        table.setColumnWidth (columnCount, Table::minimum);
-        table.setColumnJustification (columnCount, Table::right);
-
-        std::string format = context.config.get ("dateformat");
-
-        std::string ended;
-        for (unsigned int row = 0; row < tasks.size(); ++row)
-        {
-          ended = tasks[row].get ("end");
-          if (ended.length ())
-          {
-            Date dt (::atoi (ended.c_str ()));
-            ended = dt.toStringWithTime (format);
-            context.hooks.trigger ("format-end_time", "end_time", ended);
-            table.addCell (row, columnCount, ended);
-          }
-        }
-      }
-
       else if (*col == "due")
       {
         table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Due");
@@ -630,6 +526,31 @@ int runCustomReport (
         }
       }
 
+      else if (*col == "depends")
+      {
+        table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Deps");
+        table.setColumnWidth (columnCount, Table::minimum);
+        table.setColumnJustification (columnCount, Table::left);
+
+        int row = 0;
+        std::vector <Task> blocked;
+        std::vector <int> blocked_ids;
+        std::string deps;
+        foreach (task, tasks)
+        {
+          dependencyGetBlocking (*task, blocked);
+          foreach (b, blocked)
+            blocked_ids.push_back (b->id);
+
+          join (deps, ",", blocked_ids);
+          blocked_ids.clear ();
+          blocked.clear ();
+
+          context.hooks.trigger ("format-depends", "depends", deps);
+          table.addCell (row++, columnCount, deps);
+        }
+      }
+
       // Common to all columns.
       // Add underline.
       if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
@@ -666,8 +587,9 @@ int runCustomReport (
                         Table::ascendingPriority :
                         Table::descendingPriority));
 
-      else if (column == "entry" || column == "start" || column == "wait" ||
-               column == "until" || column == "end")
+      else if (column == "entry"      || column == "start"    || column == "wait"       ||
+               column == "until"      || column == "end"      || column == "entry_time" ||
+               column == "start_time" || column == "end_time")
         table.sortOn (columnIndex[column],
                       (direction == '+' ?
                         Table::ascendingDate :
@@ -741,8 +663,8 @@ int runCustomReport (
       if (maxrows)
         out << ", " << maxrows << " shown";
 
-      if (maxlines)
-        out << ", truncated to " << maxlines-1 << " lines";
+      if (maxlines && maxlines < table.rowCount ())
+        out << ", truncated to " << maxlines - 1 << " lines";
 
       out << std::endl;
     }
@@ -774,11 +696,11 @@ void validReportColumns (const std::vector <std::string>& columns)
         *it != "priority"             &&
         *it != "priority_long"        &&
         *it != "entry"                &&
-        *it != "entry_time"           &&
+        *it != "entry_time"           &&  // TODO Deprecated
         *it != "start"                &&
-        *it != "start_time"           &&
+        *it != "start_time"           &&  // TODO Deprecated
         *it != "end"                  &&
-        *it != "end_time"             &&
+        *it != "end_time"             &&  // TODO Deprecated
         *it != "due"                  &&
         *it != "countdown"            &&
         *it != "countdown_compact"    &&
@@ -791,14 +713,15 @@ void validReportColumns (const std::vector <std::string>& columns)
         *it != "tag_indicator"        &&
         *it != "description_only"     &&
         *it != "description"          &&
-        *it != "wait")
+        *it != "wait"                 &&
+        *it != "depends")
       bad.push_back (*it);
 
   if (bad.size ())
   {
     std::string error;
     join (error, ", ", bad);
-    throw std::string ("Unrecognized column name: ") + error;
+    throw std::string ("Unrecognized column name: ") + error + ".";
   }
 }
 
@@ -811,6 +734,12 @@ void validSortColumns (
   std::vector <std::string>::const_iterator sc;
   for (sc = sortColumns.begin (); sc != sortColumns.end (); ++sc)
   {
+    char direction = (*sc)[sc->length () - 1];
+    if (direction != '-' && direction != '+')
+      throw std::string ("Sort column '") +
+            *sc +
+            "' does not have a +/- ascending/descending indicator.";
+
     std::vector <std::string>::const_iterator co;
     for (co = columns.begin (); co != columns.end (); ++co)
       if (sc->substr (0, sc->length () - 1) == *co)
@@ -824,7 +753,7 @@ void validSortColumns (
   {
     std::string error;
     join (error, ", ", bad);
-    throw std::string ("Sort column is not part of the report: ") + error;
+    throw std::string ("Sort column is not part of the report: ") + error + ".";
   }
 }
 
