@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -112,52 +112,52 @@ static std::string formatDate (
 static std::string formatTask (Task task)
 {
   std::stringstream before;
-  before << "# The 'task edit <id>' command allows you to modify all aspects of a task" << std::endl
-         << "# using a text editor.  What is shown below is a representation of the"    << std::endl
-         << "# task in all it's detail.  Modify what you wish, and if you save and"     << std::endl
-         << "# quit your editor, task will read this file and try to make sense of"     << std::endl
-         << "# what changed, and apply those changes.  If you quit your editor without" << std::endl
-         << "# saving or making any modifications, task will do nothing."               << std::endl
-         << "#"                                                                         << std::endl
-         << "# Lines that begin with # represent data you cannot change, like ID."      << std::endl
-         << "# If you get too 'creative' with your editing, task will dump you back "   << std::endl
-         << "# into the editor to try again."                                           << std::endl
-         << "#"                                                                         << std::endl
-         << "# Should you find yourself in an endless Groundhog Day loop, editing and"  << std::endl
-         << "# editing the same file, just quit the editor without making any changes." << std::endl
-         << "# Task will notice this and stop the editing."                             << std::endl
-         << "#"                                                                         << std::endl
-         << "# Name               Editable details"                                     << std::endl
-         << "# -----------------  ----------------------------------------------------" << std::endl
-         << "# ID:                " << task.id                                          << std::endl
-         << "# UUID:              " << task.get ("uuid")                                << std::endl
-         << "# Status:            " << ucFirst (Task::statusToText (task.getStatus ())) << std::endl
-         << "# Mask:              " << task.get ("mask")                                << std::endl
-         << "# iMask:             " << task.get ("imask")                               << std::endl
-         << "  Project:           " << task.get ("project")                             << std::endl
-         << "  Priority:          " << task.get ("priority")                            << std::endl;
+  before << "# The 'task edit <id>' command allows you to modify all aspects of a task\n"
+         << "# using a text editor.  What is shown below is a representation of the\n"
+         << "# task in all it's detail.  Modify what you wish, and if you save and\n"
+         << "# quit your editor, taskwarrior will read this file and try to make sense\n"
+         << "# of what changed, and apply those changes.  If you quit your editor\n"
+         << "# without saving or making any modifications, taskwarrior will do nothing.\n"
+         << "#\n"
+         << "# Lines that begin with # represent data you cannot change, like ID.\n"
+         << "# If you get too 'creative' with your editing, taskwarrior will dump you\n"
+         << "# back into the editor to try again.\n"
+         << "#\n"
+         << "# Should you find yourself in an endless Groundhog Day loop, editing and\n"
+         << "# editing the same file, just quit the editor without making any changes.\n"
+         << "# Taskwarrior will notice this and stop the editing.\n"
+         << "#\n"
+         << "# Name               Editable details\n"
+         << "# -----------------  ----------------------------------------------------\n"
+         << "# ID:                " << task.id                                          << "\n"
+         << "# UUID:              " << task.get ("uuid")                                << "\n"
+         << "# Status:            " << ucFirst (Task::statusToText (task.getStatus ())) << "\n"
+         << "# Mask:              " << task.get ("mask")                                << "\n"
+         << "# iMask:             " << task.get ("imask")                               << "\n"
+         << "  Project:           " << task.get ("project")                             << "\n"
+         << "  Priority:          " << task.get ("priority")                            << "\n";
 
   std::vector <std::string> tags;
   task.getTags (tags);
   std::string allTags;
   join (allTags, " ", tags);
-  before << "# Separate the tags with spaces, like this: tag1 tag2"                     << std::endl
-         << "  Tags:              " << allTags                                          << std::endl
-         << "# The description field is allowed to wrap and use multiple lines.  Task"  << std::endl
-         << "# will combine them."                                                      << std::endl
-         << "  Description:       " << task.get ("description")                         << std::endl
-         << "  Created:           " << formatDate (task, "entry")                       << std::endl
-         << "  Started:           " << formatDate (task, "start")                       << std::endl
-         << "  Ended:             " << formatDate (task, "end")                         << std::endl
-         << "  Due:               " << formatDate (task, "due")                         << std::endl
-         << "  Until:             " << formatDate (task, "until")                       << std::endl
-         << "  Recur:             " << task.get ("recur")                               << std::endl
-         << "  Wait until:        " << formatDate (task, "wait")                        << std::endl
-         << "  Parent:            " << task.get ("parent")                              << std::endl
-         << "  Foreground color:  " << task.get ("fg")                                  << std::endl
-         << "  Background color:  " << task.get ("bg")                                  << std::endl
-         << "# Annotations look like this: <date> <text>, and there can be any number"  << std::endl
-         << "# of them."                                                                << std::endl;
+  before << "# Separate the tags with spaces, like this: tag1 tag2\n"
+         << "  Tags:              " << allTags                                          << "\n"
+         << "# The description field is allowed to wrap and use multiple lines.  Task\n"
+         << "# will combine them.\n"
+         << "  Description:       " << task.get ("description")                         << "\n"
+         << "  Created:           " << formatDate (task, "entry")                       << "\n"
+         << "  Started:           " << formatDate (task, "start")                       << "\n"
+         << "  Ended:             " << formatDate (task, "end")                         << "\n"
+         << "  Due:               " << formatDate (task, "due")                         << "\n"
+         << "  Until:             " << formatDate (task, "until")                       << "\n"
+         << "  Recur:             " << task.get ("recur")                               << "\n"
+         << "  Wait until:        " << formatDate (task, "wait")                        << "\n"
+         << "  Parent:            " << task.get ("parent")                              << "\n"
+         << "  Foreground color:  " << task.get ("fg")                                  << "\n"
+         << "  Background color:  " << task.get ("bg")                                  << "\n"
+         << "# Annotations look like this: <date> -- <text> and there can be any number of them\n"
+         << "# ' -- ' is the separator between the date and text field. It should not be removed\n";
 
   std::vector <Att> annotations;
   task.getAnnotations (annotations);
@@ -165,13 +165,21 @@ static std::string formatTask (Task task)
   {
     Date dt (::atoi (anno->name ().substr (11).c_str ()));
     before << "  Annotation:        " << dt.toString (context.config.get ("dateformat.annotation"))
-           << " "                     << anno->value ()                                 << std::endl;
+           << " -- "                  << anno->value ()                                 << "\n";
   }
 
   Date now;
-  before << "  Annotation:        " << now.toString (context.config.get ("dateformat.annotation")) << " " << std::endl
-         << "# End"                                                                     << std::endl;
+  before << "  Annotation:        " << now.toString (context.config.get ("dateformat.annotation")) << " -- \n";
 
+  // Add dependencies here.
+  std::vector <int> dependencies;
+  task.getDependencies (dependencies);
+  std::string allDeps;
+  join (allDeps, ",", dependencies);
+  before << "# Dependencies should be a comma-separated list of task IDs, with no spaces.\n"
+         << "  Dependencies:      " << allDeps << "\n";
+
+  before << "# End\n";
   return before.str ();
 }
 
@@ -184,12 +192,12 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (value != "")
     {
-      std::cout << "Project modified." << std::endl;
+      std::cout << "Project modified.\n";
       task.set ("project", value);
     }
     else
     {
-      std::cout << "Project deleted." << std::endl;
+      std::cout << "Project deleted.\n";
       task.remove ("project");
     }
   }
@@ -202,13 +210,13 @@ static void parseTask (Task& task, const std::string& after)
     {
       if (Att::validNameValue ("priority", "", value))
       {
-        std::cout << "Priority modified." << std::endl;
+        std::cout << "Priority modified.\n";
         task.set ("priority", value);
       }
     }
     else
     {
-      std::cout << "Priority deleted." << std::endl;
+      std::cout << "Priority deleted.\n";
       task.remove ("priority");
     }
   }
@@ -226,7 +234,7 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (value != "")
     {
-      std::cout << "Description modified." << std::endl;
+      std::cout << "Description modified.\n";
       task.set ("description", value);
     }
     else
@@ -242,12 +250,12 @@ static void parseTask (Task& task, const std::string& after)
     Date original (::atoi (task.get ("entry").c_str ()));
     if (!original.sameDay (edited))
     {
-      std::cout << "Creation date modified." << std::endl;
+      std::cout << "Creation date modified.\n";
       task.set ("entry", value);
     }
   }
   else
-    throw std::string ("Cannot remove creation date");
+    throw std::string ("Cannot remove creation date.");
 
   // start
   value = findDate (after, "Started:");
@@ -260,13 +268,13 @@ static void parseTask (Task& task, const std::string& after)
       Date original (::atoi (task.get ("start").c_str ()));
       if (!original.sameDay (edited))
       {
-        std::cout << "Start date modified." << std::endl;
+        std::cout << "Start date modified.\n";
         task.set ("start", value);
       }
     }
     else
     {
-      std::cout << "Start date modified." << std::endl;
+      std::cout << "Start date modified.\n";
       task.set ("start", value);
     }
   }
@@ -274,7 +282,7 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (task.get ("start") != "")
     {
-      std::cout << "Start date removed." << std::endl;
+      std::cout << "Start date removed.\n";
       task.remove ("start");
     }
   }
@@ -290,7 +298,7 @@ static void parseTask (Task& task, const std::string& after)
       Date original (::atoi (task.get ("end").c_str ()));
       if (!original.sameDay (edited))
       {
-        std::cout << "Done date modified." << std::endl;
+        std::cout << "Done date modified.\n";
         task.set ("end", value);
       }
     }
@@ -301,7 +309,7 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (task.get ("end") != "")
     {
-      std::cout << "Done date removed." << std::endl;
+      std::cout << "Done date removed.\n";
       task.setStatus (Task::pending);
       task.remove ("end");
     }
@@ -318,13 +326,13 @@ static void parseTask (Task& task, const std::string& after)
       Date original (::atoi (task.get ("due").c_str ()));
       if (!original.sameDay (edited))
       {
-        std::cout << "Due date modified." << std::endl;
+        std::cout << "Due date modified.\n";
         task.set ("due", value);
       }
     }
     else
     {
-      std::cout << "Due date modified." << std::endl;
+      std::cout << "Due date modified.\n";
       task.set ("due", value);
     }
   }
@@ -335,11 +343,11 @@ static void parseTask (Task& task, const std::string& after)
       if (task.getStatus () == Task::recurring ||
           task.get ("parent") != "")
       {
-        std::cout << "Cannot remove a due date from a recurring task." << std::endl;
+        std::cout << "Cannot remove a due date from a recurring task.\n";
       }
       else
       {
-        std::cout << "Due date removed." << std::endl;
+        std::cout << "Due date removed.\n";
         task.remove ("due");
       }
     }
@@ -356,13 +364,13 @@ static void parseTask (Task& task, const std::string& after)
       Date original (::atoi (task.get ("until").c_str ()));
       if (!original.sameDay (edited))
       {
-        std::cout << "Until date modified." << std::endl;
+        std::cout << "Until date modified.\n";
         task.set ("until", value);
       }
     }
     else
     {
-      std::cout << "Until date modified." << std::endl;
+      std::cout << "Until date modified.\n";
       task.set ("until", value);
     }
   }
@@ -370,7 +378,7 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (task.get ("until") != "")
     {
-      std::cout << "Until date removed." << std::endl;
+      std::cout << "Until date removed.\n";
       task.remove ("until");
     }
   }
@@ -384,7 +392,7 @@ static void parseTask (Task& task, const std::string& after)
       Duration d;
       if (d.valid (value))
       {
-        std::cout << "Recurrence modified." << std::endl;
+        std::cout << "Recurrence modified.\n";
         if (task.get ("due") != "")
         {
           task.set ("recur", value);
@@ -398,7 +406,7 @@ static void parseTask (Task& task, const std::string& after)
     }
     else
     {
-      std::cout << "Recurrence removed." << std::endl;
+      std::cout << "Recurrence removed.\n";
       task.setStatus (Task::pending);
       task.remove ("recur");
       task.remove ("until");
@@ -418,13 +426,13 @@ static void parseTask (Task& task, const std::string& after)
       Date original (::atoi (task.get ("wait").c_str ()));
       if (!original.sameDay (edited))
       {
-        std::cout << "Wait date modified." << std::endl;
+        std::cout << "Wait date modified.\n";
         task.set ("wait", value);
       }
     }
     else
     {
-      std::cout << "Wait date modified." << std::endl;
+      std::cout << "Wait date modified.\n";
       task.set ("wait", value);
     }
   }
@@ -432,7 +440,7 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (task.get ("wait") != "")
     {
-      std::cout << "Wait date removed." << std::endl;
+      std::cout << "Wait date removed.\n";
       task.remove ("wait");
     }
   }
@@ -443,12 +451,12 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (value != "")
     {
-      std::cout << "Parent UUID modified." << std::endl;
+      std::cout << "Parent UUID modified.\n";
       task.set ("parent", value);
     }
     else
     {
-      std::cout << "Parent UUID removed." << std::endl;
+      std::cout << "Parent UUID removed.\n";
       task.remove ("parent");
     }
   }
@@ -459,12 +467,12 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (value != "")
     {
-      std::cout << "Foreground color modified." << std::endl;
+      std::cout << "Foreground color modified.\n";
       task.set ("fg", value);
     }
     else
     {
-      std::cout << "Foreground color removed." << std::endl;
+      std::cout << "Foreground color removed.\n";
       task.remove ("fg");
     }
   }
@@ -475,12 +483,12 @@ static void parseTask (Task& task, const std::string& after)
   {
     if (value != "")
     {
-      std::cout << "Background color modified." << std::endl;
+      std::cout << "Background color modified.\n";
       task.set ("bg", value);
     }
     else
     {
-      std::cout << "Background color removed." << std::endl;
+      std::cout << "Background color removed.\n";
       task.remove ("bg");
     }
   }
@@ -499,7 +507,7 @@ static void parseTask (Task& task, const std::string& after)
         found,
         eol - found), "\t ");
 
-      std::string::size_type gap = value.find (" ");
+      std::string::size_type gap = value.find (" -- ");
       if (gap != std::string::npos)
       {
         Date when (value.substr (0, gap), context.config.get ("dateformat.annotation"));
@@ -511,13 +519,26 @@ static void parseTask (Task& task, const std::string& after)
 
         std::stringstream name;
         name << "annotation_" << when.toEpoch ();
-        std::string text = trim (value.substr (gap), "\t ");
+        std::string text = trim (value.substr (gap + 4), "\t ");
         annotations.push_back (Att (name.str (), text));
       }
     }
   }
 
   task.setAnnotations (annotations);
+
+  // Dependencies
+  value = findValue (after, "Dependencies:");
+  std::vector <std::string> dependencies;
+  split (dependencies, value, ",");
+
+  task.remove ("depends");
+  foreach (dep, dependencies)
+  {
+    int id = atoi (dep->c_str ());
+    if (id)
+      task.addDependency (id);
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -530,10 +551,13 @@ void editFile (Task& task)
 
   // Create a temp file name in data.location.
   std::stringstream file;
-  file << location.data << "/task." << getpid () << "." << task.id << ".task";
+  file << "task." << getpid () << "." << task.id << ".task";
+  std::string path = location.data + "/" + file.str ();
 
   // Format the contents, T -> text, write to a file.
   std::string before = formatTask (task);
+  int ignored = chdir (location.data.c_str ());
+  ++ignored; // Keep compiler quiet.
   File::write (file.str (), before);
 
   // Determine correct editor: .taskrc:editor > $VISUAL > $EDITOR > vi
@@ -550,11 +574,11 @@ void editFile (Task& task)
 
 ARE_THESE_REALLY_HARMFUL:
   // Launch the editor.
-  std::cout << "Launching '" << editor << "' now..." << std::endl;
+  std::cout << "Launching '" << editor << "' now...\n";
   if (-1 == system (editor.c_str ()))
-    std::cout << "No editing performed." << std::endl;
+    std::cout << "No editing performed.\n";
   else
-    std::cout << "Editing complete." << std::endl;
+    std::cout << "Editing complete.\n";
 
   // Slurp file.
   std::string after;
@@ -564,7 +588,7 @@ ARE_THESE_REALLY_HARMFUL:
   // if changes were made.
   if (before != after)
   {
-    std::cout << "Edits were detected." << std::endl;
+    std::cout << "Edits were detected.\n";
     std::string problem = "";
     bool oops = false;
 
@@ -581,18 +605,18 @@ ARE_THESE_REALLY_HARMFUL:
 
     if (oops)
     {
-      std::cout << "Error: " << problem << std::endl;
+      std::cout << "Error: " << problem << "\n";
 
       // Preserve the edits.
       before = after;
       File::write (file.str (), before);
 
-      if (confirm ("Task couldn't handle your edits.  Would you like to try again?"))
+      if (confirm ("Taskwarrior couldn't handle your edits.  Would you like to try again?"))
         goto ARE_THESE_REALLY_HARMFUL;
     }
   }
   else
-    std::cout << "No edits were detected." << std::endl;
+    std::cout << "No edits were detected.\n";
 
   // Cleanup.
   File::remove (file.str ());
@@ -602,7 +626,7 @@ ARE_THESE_REALLY_HARMFUL:
 // Introducing the Silver Bullet.  This feature is the catch-all fixative for
 // various other ills.  This is like opening up the hood and going in with a
 // wrench.  To be used sparingly.
-int handleEdit (std::string &outs)
+int handleEdit (std::string& outs)
 {
   int rc = 0;
 
@@ -625,6 +649,9 @@ int handleEdit (std::string &outs)
       editFile (*task);
       context.tdb.update (*task);
 /*
+      TODO Figure out what this is.  I can't remember, but don't want to remove
+           it until I do.
+
       foreach (other, all)
       {
         if (other->id != task.id) // Don't edit the same task again.
