@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -28,11 +28,13 @@
 #include <sstream>
 #include <stdio.h>
 #include <unistd.h>
-#include "File.h"
-#include "Date.h"
-#include "text.h"
-#include "util.h"
-#include "main.h"
+#include <Nibbler.h>
+#include <File.h>
+#include <Date.h>
+#include <text.h>
+#include <util.h>
+#include <main.h>
+#include "Transport.h"
 
 extern Context context;
 
@@ -46,6 +48,7 @@ enum fileType
   task_cmd_line,
   todo_sh_2_0,
   csv,
+  yaml,
   text
 };
 
@@ -77,6 +80,13 @@ static fileType determineFileType (const std::vector <std::string>& lines)
       return task_1_4_3;
   }
 
+  if (lines.size () > 2 &&
+      lines[0] == "%YAML 1.1" &&
+      lines[1] == "---")
+  {
+    return yaml;
+  }
+
   // A task command line might include a priority or project.
   for (unsigned int i = 0; i < lines.size (); ++i)
   {
@@ -105,16 +115,16 @@ static fileType determineFileType (const std::vector <std::string>& lines)
     // All done tasks begin with "x YYYY-MM-DD".
     if (lines[i].length () > 12)
     {
-      if (           lines[i][0] == 'x' &&
-                     lines[i][1] == ' ' &&
+      if (lines[i][0] == 'x' &&
+          lines[i][1] == ' ' &&
           isdigit (lines[i][2]) &&
           isdigit (lines[i][3]) &&
           isdigit (lines[i][4]) &&
           isdigit (lines[i][5]) &&
-                     lines[i][6] == '-' &&
+          lines[i][6] == '-' &&
           isdigit (lines[i][7]) &&
           isdigit (lines[i][8]) &&
-                     lines[i][9] == '-' &&
+          lines[i][9] == '-' &&
           isdigit (lines[i][10]) &&
           isdigit (lines[i][11]))
         return todo_sh_2_0;
@@ -153,7 +163,7 @@ static fileType determineFileType (const std::vector <std::string>& lines)
     return csv;
 
   // Looks like 'text' is the default case, if there is any data at all.
-  if (lines.size () > 1)
+  if (lines.size () >= 1)
     return text;
 
   return not_a_clue;
@@ -162,9 +172,12 @@ static fileType determineFileType (const std::vector <std::string>& lines)
 ////////////////////////////////////////////////////////////////////////////////
 static void decorateTask (Task& task)
 {
-  char entryTime[16];
-  sprintf (entryTime, "%u", (unsigned int) time (NULL));
-  task.set ("entry", entryTime);
+  if (!task.has ("entry"))
+  {
+    char entryTime[16];
+    sprintf (entryTime, "%u", (unsigned int) time (NULL));
+    task.set ("entry", entryTime);
+  }
 
   task.setStatus (Task::pending);
 
@@ -325,14 +338,13 @@ static std::string importTask_1_4_3 (const std::vector <std::string>& lines)
       << (lines.size () - failed.size () - 1)
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
@@ -487,14 +499,13 @@ static std::string importTask_1_5_0 (const std::vector <std::string>& lines)
       << (lines.size () - failed.size () - 1)
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
@@ -650,14 +661,13 @@ static std::string importTask_1_6_0 (const std::vector <std::string>& lines)
       << (lines.size () - failed.size () - 1)
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
@@ -698,14 +708,13 @@ static std::string importTaskCmdLine (const std::vector <std::string>& lines)
       << (lines.size () - failed.size ())
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
@@ -789,6 +798,10 @@ static std::string importTodoSh_2_0 (const std::vector <std::string>& lines)
       context.parse ();
       decorateTask (context.task);
 
+      // Override the Task::pending that decorateTask applies.
+      if (!isPending)
+        context.task.setStatus (Task::completed);
+
       context.task.set ("uuid", uuid ());
 
       if (isPending)
@@ -823,14 +836,13 @@ static std::string importTodoSh_2_0 (const std::vector <std::string>& lines)
       << (lines.size () - failed.size ())
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
   return out.str ();
 }
@@ -887,14 +899,13 @@ static std::string importText (const std::vector <std::string>& lines)
       << count
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
@@ -1066,6 +1077,7 @@ static std::string importCSV (const std::vector <std::string>& lines)
       {
         std::string value = lowerCase (unquoteText (trim (fields[f])));
 
+        // TODO What happened to "pending"?
              if (value == "recurring") task.setStatus (Task::recurring);
         else if (value == "deleted")   task.setStatus (Task::deleted);
         else if (value == "completed") task.setStatus (Task::completed);
@@ -1132,30 +1144,144 @@ static std::string importCSV (const std::vector <std::string>& lines)
       << (lines.size () - failed.size () - 1)
       << " tasks successfully, with "
       << failed.size ()
-      << " errors."
-      << std::endl;
+      << " errors.\n";
 
   if (failed.size ())
   {
     std::string bad;
     join (bad, "\n", failed);
-    return out.str () + "\nCould not import:\n\n" + bad;
+    return out.str () + "\nCould not import:\n\n" + bad + "\n";
   }
 
   return out.str ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleImport (std::string &outs)
+static std::string importYAML (const std::vector <std::string>& lines)
+{
+  int count = 0;
+
+  context.tdb.lock (context.config.getBoolean ("locking"));
+
+  // Load all the tasks so that the uuid uniqueness can be checked.
+  std::vector <Task> tasks;
+  Filter filter;
+  context.tdb.load (tasks, filter);
+
+  Task t;
+
+  std::string name;
+  std::string value;
+
+  bool inAnno = false;
+  std::string annoEntry;
+  Task::status status = Task::pending;
+
+  std::vector <std::string>::const_iterator it;
+  for (it = lines.begin (); it != lines.end (); ++it)
+  {
+    name = "";
+    value = "";
+
+    Nibbler n (*it);
+    n.skipWS ();
+    n.getUntil (':', name);
+    n.skip (':');
+    n.skipWS ();
+    n.getUntilEOL (value);
+
+    if (name == "%YAML 1.1")
+      ;
+
+    else if (name == "---")
+      ;
+
+    else if (name == "task" || name == "...")
+    {
+      if (t.size ())
+      {
+        // Generate a UUID if not present.
+        if (t.get ("uuid") == "")
+          t.set ("uuid", uuid ());
+
+        // Add defaults.
+        decorateTask (t);
+        t.setStatus (status);
+
+        // TODO Fail on UUID collision.
+
+        context.tdb.add (t);
+        t.clear ();
+        ++count;
+      }
+    }
+
+    else if (name == "status")
+    {
+           if (value == "waiting")   status = Task::waiting;
+      else if (value == "completed") status = Task::completed;
+      else if (value == "deleted")   status = Task::deleted;
+      else if (value == "recurring") status = Task::recurring;
+      else                           status = Task::pending;
+    }
+
+    else if (name == "annotation")
+      inAnno = true;
+
+    else if (name == "entry" && inAnno)
+      annoEntry = value;
+
+    else if (name == "description" && inAnno)
+    {
+      t.set ("annotation_" + annoEntry, value);
+      annoEntry = "";
+      inAnno = false;
+    }
+
+    else if (name != "" && value != "")
+      t.set (name, value);
+  }
+
+  context.tdb.commit ();
+  context.tdb.unlock ();
+
+  std::stringstream out;
+  out << "Imported "
+      << count
+      << " tasks successfully.\n";
+
+  return out.str ();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+int handleImport (std::string& outs)
 {
   int rc = 0;
 
   if (context.hooks.trigger ("pre-import-command"))
   {
-    std::stringstream out;
-
-    // Use the description as a file name.
+    std::stringstream out;		
+		
+		// Use the description as a file name.
     std::string file = trim (context.task.get ("description"));
+		
+#if FEATURE_URL > 0
+    std::string tmpfile = "";
+    Uri uri (file);
+    uri.parse ();
+    
+    Transport* transport;
+    if ((transport = Transport::getTransport (uri)) != NULL )
+    {
+      std::string location (context.config.get ("data.location"));
+      tmpfile = location + "/import.data";
+      transport->recv (tmpfile);
+      delete transport;
+
+      file = tmpfile;
+    }
+#endif
+    
     if (file.length () > 0)
     {
       // Load the file.
@@ -1167,12 +1293,6 @@ int handleImport (std::string &outs)
       for (it = all.begin (); it != all.end (); ++it)
       {
         std::string line = *it;
-
-        // Strip comments
-        std::string::size_type pound = line.find ("#");
-        if (pound != std::string::npos)
-          line = line.substr (0, pound);
-
         trim (line);
 
         // Skip blank lines
@@ -1185,22 +1305,23 @@ int handleImport (std::string &outs)
       std::string identifier;
       switch (type)
       {
-      case task_1_4_3:    identifier = "This looks like an older task export file.";              break;
-      case task_1_5_0:    identifier = "This looks like a recent task export file.";              break;
-      case task_1_6_0:    identifier = "This looks like a current task export file.";             break;
-      case task_cmd_line: identifier = "This looks like task command line arguments.";            break;
-      case todo_sh_2_0:   identifier = "This looks like a todo.sh 2.x file.";                     break;
-      case csv:           identifier = "This looks like a CSV file, but not a task export file."; break;
-      case text:          identifier = "This looks like a text file with one task per line.";     break;
+      case task_1_4_3:    identifier = "This looks like an older taskwarrior export file.";              break;
+      case task_1_5_0:    identifier = "This looks like a recent taskwarrior export file.";              break;
+      case task_1_6_0:    identifier = "This looks like a current taskwarrior export file.";             break;
+      case task_cmd_line: identifier = "This looks like taskwarrior command line arguments.";            break;
+      case todo_sh_2_0:   identifier = "This looks like a todo.sh 2.x file.";                            break;
+      case csv:           identifier = "This looks like a CSV file, but not a taskwarrior export file."; break;
+      case yaml:          identifier = "This looks like a YAML file.";                                   break;
+      case text:          identifier = "This looks like a text file with one task per line.";            break;
       case not_a_clue:
-        throw std::string ("Task cannot determine which type of file this is, "
-                           "and cannot proceed.");
+        throw std::string ("Taskwarrior cannot determine which type of file "
+                           "this is, and cannot proceed.");
       }
 
       // For tty users, confirm the import, as it is destructive.
       if (isatty (fileno (stdout)))
         if (! confirm (identifier + "  Okay to proceed?"))
-          throw std::string ("Task will not import any data.");
+          throw std::string ("Taskwarrior will not import any data.");
 
       // Determine which type it might be, then attempt an import.
       switch (type)
@@ -1211,9 +1332,17 @@ int handleImport (std::string &outs)
       case task_cmd_line: out << importTaskCmdLine (lines); break;
       case todo_sh_2_0:   out << importTodoSh_2_0  (lines); break;
       case csv:           out << importCSV         (lines); break;
+      case yaml:          out << importYAML        (lines); break;
       case text:          out << importText        (lines); break;
       case not_a_clue:    /* to stop the compiler from complaining. */ break;
       }
+
+#if FEATURE_URL > 0
+      if (tmpfile != "")
+      {
+        remove (tmpfile.c_str ());
+      }
+#endif
     }
     else
       throw std::string ("You must specify a file to import.");
