@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -27,7 +27,13 @@
 
 #include <iostream>
 #include <stdlib.h>
+
+#ifdef CYGWIN
+#include <time.h>
+#else
 #include <sys/time.h>
+#endif
+
 #include "Context.h"
 #include "../auto.h"
 
@@ -36,12 +42,20 @@ Context context;
 int main (int argc, char** argv)
 {
   // Set up randomness.
+#ifdef CYGWIN
+  #ifdef HAVE_SRANDOM
+  srandom (time (NULL));
+  #else
+  srand (time (NULL));
+  #endif
+#else
+  #ifdef HAVE_SRANDOM
   struct timeval tv;
   gettimeofday (&tv, NULL);
-#ifdef HAVE_SRANDOM
   srandom (tv.tv_usec);
-#else
+  #else
   srand (tv.tv_usec);
+  #endif
 #endif
 
   int status = 0;
@@ -50,6 +64,22 @@ int main (int argc, char** argv)
   {
     context.initialize (argc, argv);
 
+/* From 2.0.0
+    std::string::size_type task = context.program.find ("/task");
+    std::string::size_type t    = context.program.find ("/t");
+    std::string::size_type cal  = context.program.find ("/cal");
+
+    if (context.program != "task"                                            &&
+        context.program != "t"                                               &&
+        context.program != "cal"                                             &&
+        (task == std::string::npos || context.program.length () != task + 5) &&
+        (t    == std::string::npos || context.program.length () != t    + 2) &&
+        (cal  == std::string::npos || context.program.length () != cal  + 4))
+      status = context.handleInteractive ();
+    else
+      status = context.run ();
+*/
+
     std::string::size_type itask = context.program.find ("/itask");
     if (context.program == "itask" ||
         (itask != std::string::npos && context.program.length () == itask + 5))
@@ -60,13 +90,13 @@ int main (int argc, char** argv)
 
   catch (std::string& error)
   {
-    std::cout << error << std::endl;
+    std::cout << error << "\n";
     return -1;
   }
 
   catch (...)
   {
-    std::cerr << context.stringtable.get (100, "Unknown error.") << std::endl;
+    std::cerr << context.stringtable.get (100, "Unknown error.") << "\n";
     return -2;
   }
 
