@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
@@ -24,10 +24,12 @@
 //     USA
 //
 ////////////////////////////////////////////////////////////////////////////////
+
 #include <iostream>
 #include <iomanip>
 #include <sstream>
 #include <fstream>
+#include <algorithm>
 #include <sys/types.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -49,10 +51,12 @@
 #include <ncurses.h>
 #endif
 
+static void countTasks (const std::vector <Task>&, const std::string&, const std::string&, int&, int&);
+
 extern Context context;
 
 ////////////////////////////////////////////////////////////////////////////////
-int shortUsage (std::string &outs)
+int shortUsage (std::string& outs)
 {
   Table table;
 
@@ -199,13 +203,34 @@ int shortUsage (std::string &outs)
   table.addCell (row, 1, "task export");
   table.addCell (row, 2, "Lists all tasks in CSV format.  Alias to export.csv");
 
+  row = table.addRow ();
+  table.addCell (row, 1, "task export.csv");
+  table.addCell (row, 2, "Lists all tasks in CSV format.");
+
   row = table.addRow ();
   table.addCell (row, 1, "task export.ical");
   table.addCell (row, 2, "Lists all tasks in iCalendar format.");
 
   row = table.addRow ();
-  table.addCell (row, 1, "task color [sample]");
-  table.addCell (row, 2, "Displays all possible colors, or a sample.");
+  table.addCell (row, 1, "task export.yaml");
+  table.addCell (row, 2, "Lists all tasks in YAML format.");
+
+  row = table.addRow ();
+  table.addCell (row, 1, "task merge URL");
+  table.addCell (row, 2, "Merges the specified undo.data file with the local data files.");
+
+  row = table.addRow ();
+  table.addCell (row, 1, "task push URL");
+  table.addCell (row, 2, "Pushes the local *.data files to the URL.");
+
+	row = table.addRow ();
+  table.addCell (row, 1, "task pull URL");
+  table.addCell (row, 2, "Overwrites the local *.data files with those found at the URL.");
+
+  row = table.addRow ();
+  table.addCell (row, 1, "task color [sample | legend]");
+  table.addCell (row, 2, "Displays all possible colors, a named sample, or a "
+                         "legend containing all currently defined colors.");
 
   row = table.addRow ();
   table.addCell (row, 1, "task version");
@@ -219,6 +244,10 @@ int shortUsage (std::string &outs)
   table.addCell (row, 1, "task config [name [value | '']]");
   table.addCell (row, 2, "Add, modify and remove settings in the task configuration.");
 
+  row = table.addRow ();
+  table.addCell (row, 1, "task diagnostics");
+  table.addCell (row, 2, "Information needed when reporting a problem.");
+
   row = table.addRow ();
   table.addCell (row, 1, "task help");
   table.addCell (row, 2, "Shows the long usage text.");
@@ -240,19 +269,19 @@ int shortUsage (std::string &outs)
 
   std::stringstream out;
   out << table.render ()
-      << std::endl
-      << "Documentation for task can be found using 'man task', 'man taskrc', "
-      << "'man task-tutorial', 'man task-color', 'man task-faq' or at "
-      << "http://taskwarrior.org"
-      << std::endl
-      << std::endl;
+      << "\n"
+      << "Documentation for taskwarrior can be found using 'man task', "
+      << "'man taskrc', 'man task-tutorial', 'man task-color', 'man task-faq' "
+      << "or at http://taskwarrior.org"
+      << "\n"
+      << "\n";
 
   outs = out.str ();
   return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int longUsage (std::string &outs)
+int longUsage (std::string& outs)
 {
   int rc = 0;
 
@@ -318,12 +347,12 @@ int longUsage (std::string &outs)
         << "  task add \"quoted ' quote\""                                      << "\n"
         << "  task add escaped \\' quote"                                       << "\n"
         <<                                                                         "\n"
-        << "The argument -- tells task to treat all other args as description." << "\n"
+        << "The argument -- tells taskwarrior to treat all other args as description." << "\n"
         << "  task add -- project:Home needs scheduling"                        << "\n"
         <<                                                                         "\n"
         << "Many characters have special meaning to the shell, including:"      << "\n"
         << "  $ ! ' \" ( ) ; \\ ` * ? { } [ ] < > | & % # ~"                    << "\n"
-        << std::endl;
+        << "\n";
 
     outs = out.str();
     context.hooks.trigger ("post-usage-command");
@@ -334,7 +363,7 @@ int longUsage (std::string &outs)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Display all information for the given task.
-int handleInfo (std::string &outs)
+int handleInfo (std::string& outs)
 {
   int rc = 0;
 
@@ -397,6 +426,10 @@ int handleInfo (std::string &outs)
       table.addCell (row, 0, "Description");
       table.addCell (row, 1, getFullDescription (*task, "info"));
 
+      Color c;
+      autoColorize (*task, c);
+      table.setCellColor (row, 1, c);
+
       // status
       row = table.addRow ();
       table.addCell (row, 0, "Status");
@@ -420,49 +453,79 @@ int handleInfo (std::string &outs)
         table.addCell (row, 1, value);
       }
 
-      if (task->getStatus () == Task::recurring ||
-          task->has ("parent"))
+      // dependencies: blocked
       {
-        // recur
-        if (task->has ("recur"))
+        std::vector <Task> blocked;
+        dependencyGetBlocking (*task, blocked);
+        if (blocked.size ())
         {
-          row = table.addRow ();
-          table.addCell (row, 0, "Recurrence");
-          value = task->get ("recur");
-          context.hooks.trigger ("format-recur", "recur", value);
-          table.addCell (row, 1, value);
-        }
+          std::stringstream message;
+          std::vector <Task>::const_iterator it;
+          for (it = blocked.begin (); it != blocked.end (); ++it)
+            message << it->id << " " << it->get ("description") << "\n";
 
-        // until
-        if (task->has ("until"))
+          row = table.addRow ();
+          table.addCell (row, 0, "This task blocked by");
+          table.addCell (row, 1, message.str ());
+        }
+      }
+
+      // dependencies: blocking
+      {
+        std::vector <Task> blocking;
+        dependencyGetBlocked (*task, blocking);
+        if (blocking.size ())
         {
+          std::stringstream message;
+          std::vector <Task>::const_iterator it;
+          for (it = blocking.begin (); it != blocking.end (); ++it)
+            message << it->id << " " << it->get ("description") << "\n";
+
           row = table.addRow ();
-          table.addCell (row, 0, "Recur until");
-
-          Date dt (atoi (task->get ("until").c_str ()));
-          std::string format = context.config.get ("reportdateformat");
-          if (format == "")
-            format = context.config.get ("dateformat");
-
-          std::string until = getDueDate (*task, format);
-          table.addCell (row, 1, until);
+          table.addCell (row, 0, "This task is blocking");
+          table.addCell (row, 1, message.str ());
         }
+      }
 
-        // mask
-        if (task->has ("mask"))
-        {
-          row = table.addRow ();
-          table.addCell (row, 0, "Mask");
-          table.addCell (row, 1, task->get ("mask"));
-        }
+      // recur
+      if (task->has ("recur"))
+      {
+        row = table.addRow ();
+        table.addCell (row, 0, "Recurrence");
+        value = task->get ("recur");
+        context.hooks.trigger ("format-recur", "recur", value);
+        table.addCell (row, 1, value);
+      }
 
+      // until
+      if (task->has ("until"))
+      {
+        row = table.addRow ();
+        table.addCell (row, 0, "Recur until");
+
+        Date dt (atoi (task->get ("until").c_str ()));
+        std::string format = context.config.get ("reportdateformat");
+        if (format == "")
+          format = context.config.get ("dateformat");
+
+        std::string until = getDueDate (*task, format);
+        table.addCell (row, 1, until);
+      }
+
+      // mask
+      if (task->getStatus () == Task::recurring)
+      {
+        row = table.addRow ();
+        table.addCell (row, 0, "Mask");
+        table.addCell (row, 1, task->get ("mask"));
+      }
+
+      if (task->has ("parent"))
+      {
         // parent
-        if (task->has ("parent"))
-        {
-          row = table.addRow ();
-          table.addCell (row, 0, "Parent task");
-          table.addCell (row, 1, task->get ("parent"));
-        }
+        row = table.addRow ();
+        table.addCell (row, 0, "Parent task");
+        table.addCell (row, 1, task->get ("parent"));
 
         // imask
         row = table.addRow ();
@@ -574,13 +637,27 @@ int handleInfo (std::string &outs)
         table.addCell (row, 1, color);
       }
 
+      // Task::urgency
+      // TODO Enable this later.  This was for testing.
+      //row = table.addRow ();
+      //table.addCell (row, 0, "Urgency");
+      //table.addCell (row, 1, task->urgency ());
+
+      // If an alternating row color is specified, notify the table.
+      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+      {
+        Color alternate (context.config.get ("color.alternate"));
+        if (alternate.nontrivial ())
+          table.setTableAlternateColor (alternate);
+      }
+
       out << optionalBlankLine ()
           << table.render ()
-          << std::endl;
+          << "\n";
     }
 
     if (! tasks.size ()) {
-      out << "No matches." << std::endl;
+      out << "No matches.\n";
       rc = 1;
     }
 
@@ -595,7 +672,7 @@ int handleInfo (std::string &outs)
 // Project  Remaining  Avg Age  Complete  0%                  100%
 // A               12      13d       55%  XXXXXXXXXXXXX-----------
 // B              109   3d 12h       10%  XXX---------------------
-int handleReportSummary (std::string &outs)
+int handleReportSummary (std::string& outs)
 {
   int rc = 0;
 
@@ -696,7 +773,7 @@ int handleReportSummary (std::string &outs)
         table.addCell (row, 0, (i->first == "" ? "(none)" : i->first));
         table.addCell (row, 1, countPending[i->first]);
         if (counter[i->first])
-          table.addCell (row, 2, Duration ((int) sumEntry[i->first] / counter[i->first]).format ());
+          table.addCell (row, 2, Duration ((int) (sumEntry[i->first] / (double)counter[i->first])).format ());
 
         int c = countCompleted[i->first];
         int p = countPending[i->first];
@@ -729,9 +806,9 @@ int handleReportSummary (std::string &outs)
           << optionalBlankLine ()
           << table.rowCount ()
           << (table.rowCount () == 1 ? " project" : " projects")
-          << std::endl;
+          << "\n";
     else {
-      out << "No projects." << std::endl;
+      out << "No projects.\n";
       rc = 1;
     }
 
@@ -742,74 +819,6 @@ int handleReportSummary (std::string &outs)
   return rc;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// A summary of the most important pending tasks.
-//
-// For every project, pull important tasks to present as an 'immediate' task
-// list.  This hides the overwhelming quantity of other tasks.
-//
-// Present at most three tasks for every project.  Select the tasks using
-// these criteria:
-//   - due:< 1wk, pri:*
-//   - due:*, pri:H
-//   - pri:H
-//   - due:*, pri:M
-//   - pri:M
-//   - due:*, pri:L
-//   - pri:L
-//   - due:*, pri:*        <-- everything else
-//
-// Make the "three" tasks a configurable number
-//
-int handleReportNext (std::string &outs)
-{
-  // Load report configuration.
-  std::string columnList = context.config.get ("report.next.columns");
-  std::string labelList  = context.config.get ("report.next.labels");
-  std::string sortList   = context.config.get ("report.next.sort");
-  std::string filterList = context.config.get ("report.next.filter");
-
-  std::vector <std::string> filterArgs;
-  split (filterArgs, filterList, ' ');
-  {
-    Cmd cmd ("next");
-    Task task;
-    Sequence sequence;
-    Subst subst;
-    Filter filter;
-    context.parse (filterArgs, cmd, task, sequence, subst, filter);
-
-    context.sequence.combine (sequence);
-
-    // Allow limit to be overridden by the command line.
-    if (!context.task.has ("limit") && task.has ("limit"))
-      context.task.set ("limit", task.get ("limit"));
-
-    foreach (att, filter)
-      context.filter.push_back (*att);
-  }
-
-  // Get all the tasks.
-  std::vector <Task> tasks;
-  context.tdb.lock (context.config.getBoolean ("locking"));
-  handleRecurrence ();
-  context.tdb.load (tasks, context.filter);
-  context.tdb.commit ();
-  context.tdb.unlock ();
-
-  // Restrict to matching subset.
-  gatherNextTasks (tasks);
-
-  return runCustomReport (
-    "next",
-    columnList,
-    labelList,
-    sortList,
-    filterList,
-    tasks,
-    outs);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Year Month    Added Completed Deleted
 // 2006 November    87        63      14
@@ -851,7 +860,7 @@ time_t yearlyEpoch (const std::string& date)
   return 0;
 }
 
-int handleReportHistoryMonthly (std::string &outs)
+int handleReportHistoryMonthly (std::string& outs)
 {
   int rc = 0;
 
@@ -1003,10 +1012,10 @@ int handleReportHistoryMonthly (std::string &outs)
     if (table.rowCount ())
       out << optionalBlankLine ()
           << table.render ()
-          << std::endl;
+          << "\n";
     else
     {
-      out << "No tasks." << std::endl;
+      out << "No tasks.\n";
       rc = 1;
     }
 
@@ -1018,7 +1027,7 @@ int handleReportHistoryMonthly (std::string &outs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportHistoryAnnual (std::string &outs)
+int handleReportHistoryAnnual (std::string& outs)
 {
   int rc = 0;
 
@@ -1167,10 +1176,10 @@ int handleReportHistoryAnnual (std::string &outs)
     if (table.rowCount ())
       out << optionalBlankLine ()
           << table.render ()
-          << std::endl;
+          << "\n";
     else
     {
-      out << "No tasks." << std::endl;
+      out << "No tasks.\n";
       rc = 1;
     }
 
@@ -1182,7 +1191,7 @@ int handleReportHistoryAnnual (std::string &outs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportGHistoryMonthly (std::string &outs)
+int handleReportGHistoryMonthly (std::string& outs)
 {
   int rc = 0;
 
@@ -1361,7 +1370,7 @@ int handleReportGHistoryMonthly (std::string &outs)
     {
       out << optionalBlankLine ()
           << table.render ()
-          << std::endl;
+          << "\n";
 
       if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
         out << "Legend: "
@@ -1371,13 +1380,13 @@ int handleReportGHistoryMonthly (std::string &outs)
             << ", "
             << color_delete.colorize ("deleted")
             << optionalBlankLine ()
-            << std::endl;
+            << "\n";
       else
-        out << "Legend: + added, X completed, - deleted" << std::endl;
+        out << "Legend: + added, X completed, - deleted\n";
     }
     else
     {
-      out << "No tasks." << std::endl;
+      out << "No tasks.\n";
       rc = 1;
     }
 
@@ -1389,7 +1398,7 @@ int handleReportGHistoryMonthly (std::string &outs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportGHistoryAnnual (std::string &outs)
+int handleReportGHistoryAnnual (std::string& outs)
 {
   int rc = 0;
 
@@ -1564,7 +1573,7 @@ int handleReportGHistoryAnnual (std::string &outs)
     {
       out << optionalBlankLine ()
           << table.render ()
-          << std::endl;
+          << "\n";
 
       if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
         out << "Legend: "
@@ -1574,13 +1583,13 @@ int handleReportGHistoryAnnual (std::string &outs)
             << ", "
             << color_delete.colorize ("deleted")
             << optionalBlankLine ()
-            << std::endl;
+            << "\n";
       else
-        out << "Legend: + added, X completed, - deleted" << std::endl;
+        out << "Legend: + added, X completed, - deleted\n";
     }
     else
     {
-      out << "No tasks." << std::endl;
+      out << "No tasks.\n";
       rc = 1;
     }
 
@@ -1592,7 +1601,7 @@ int handleReportGHistoryAnnual (std::string &outs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportTimesheet (std::string &outs)
+int handleReportTimesheet (std::string& outs)
 {
   int rc = 0;
 
@@ -1642,9 +1651,9 @@ int handleReportTimesheet (std::string &outs)
                           + endString.toString (context.config.get ("dateformat"));
 
       Color bold (Color::nocolor, Color::nocolor, false, true, false);
-      out << std::endl
+      out << "\n"
           << (color ? bold.colorize (title) : title)
-          << std::endl;
+          << "\n";
 
       // Render the completed table.
       Table completed;
@@ -1701,11 +1710,11 @@ int handleReportTimesheet (std::string &outs)
         }
       }
 
-      out << "  Completed (" << completed.rowCount () << " tasks)" << std::endl;
+      out << "  Completed (" << completed.rowCount () << " tasks)\n";
 
       if (completed.rowCount ())
         out << completed.render ()
-            << std::endl;
+            << "\n";
 
       // Now render the started table.
       Table started;
@@ -1717,12 +1726,12 @@ int handleReportTimesheet (std::string &outs)
 
       if (color && context.config.getBoolean ("fontunderline"))
       {
-        completed.setColumnUnderline (1);
-        completed.setColumnUnderline (2);
-        completed.setColumnUnderline (3);
+        started.setColumnUnderline (1);
+        started.setColumnUnderline (2);
+        started.setColumnUnderline (3);
       }
       else
-        completed.setTableDashedUnderline ();
+        started.setTableDashedUnderline ();
 
       started.setColumnWidth (0, Table::minimum);
       started.setColumnWidth (1, Table::minimum);
@@ -1762,12 +1771,11 @@ int handleReportTimesheet (std::string &outs)
         }
       }
 
-      out << "  Started (" << started.rowCount () << " tasks)" << std::endl;
+      out << "  Started (" << started.rowCount () << " tasks)\n";
 
       if (started.rowCount ())
         out << started.render ()
-            << std::endl
-            << std::endl;
+            << "\n\n";
 
       // Prior week.
       start -= 7 * 86400;
@@ -1960,6 +1968,7 @@ std::string renderMonths (
           foreach (task, all)
           {
             if (task->getStatus () == Task::pending &&
+                !task->hasTag ("nocal")             &&
                 task->has ("due"))
             {
               std::string due = task->get ("due");
@@ -2007,7 +2016,7 @@ std::string renderMonths (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportCalendar (std::string &outs)
+int handleReportCalendar (std::string& outs)
 {
   int rc = 0;
 
@@ -2040,54 +2049,103 @@ int handleReportCalendar (std::string &outs)
     int mTo = mFrom;
     int yTo = yFrom;
 
-    // Determine what to do
-    int numberOfArgs = context.args.size();
+    // Defaults.
+    monthsToDisplay = monthsPerLine;
+    mFrom = today.month ();
+    yFrom = today.year ();
 
-    if (numberOfArgs == 1) {
-      // task cal
-      monthsToDisplay = monthsPerLine;
-      mFrom = today.month();
-      yFrom = today.year();
-    }
-    else if (numberOfArgs == 2) {
-      if (context.args[1] == "y") {
-        // task cal y
-        monthsToDisplay = 12;
-        mFrom = today.month();
-        yFrom = today.year();
-      }
-      else if (context.args[1] == "due") {
-        // task cal due
-        monthsToDisplay = monthsPerLine;
+    // Set up a vector of commands (1), for autoComplete.
+    std::vector <std::string> commandNames;
+    commandNames.push_back ("calendar");
+
+    // Set up a vector of keywords, for autoComplete.
+    std::vector <std::string> keywordNames;
+    keywordNames.push_back ("due");
+
+    // Set up a vector of months, for autoComplete.
+    std::vector <std::string> monthNames;
+    monthNames.push_back ("january");
+    monthNames.push_back ("february");
+    monthNames.push_back ("march");
+    monthNames.push_back ("april");
+    monthNames.push_back ("may");
+    monthNames.push_back ("june");
+    monthNames.push_back ("july");
+    monthNames.push_back ("august");
+    monthNames.push_back ("september");
+    monthNames.push_back ("october");
+    monthNames.push_back ("november");
+    monthNames.push_back ("december");
+
+    // For autoComplete results.
+    std::vector <std::string> matches;
+
+    // Look at all args, regardless of sequence.
+    int argMonth = 0;
+    int argYear = 0;
+    bool argWholeYear = false;
+    foreach (arg, context.args)
+    {
+      // Some version of "calendar".
+      if (autoComplete (lowerCase (*arg), commandNames, matches) == 1)
+        continue;
+
+      // "due".
+      else if (autoComplete (lowerCase (*arg), keywordNames, matches) == 1)
         getpendingdate = true;
+
+      // "y".
+      else if (lowerCase (*arg) == "y")
+        argWholeYear = true;
+
+      // YYYY.
+      else if (digitsOnly (*arg) && arg->length () == 4)
+        argYear = atoi (arg->c_str ());
+
+      // MM.
+      else if (digitsOnly (*arg) && arg->length () <= 2)
+      {
+        argMonth = atoi (arg->c_str ());
+        if (argMonth < 1 || argMonth > 12)
+          throw std::string ("Argument '") + *arg + "' is not a valid month.";
       }
-      else {
-        // task cal 2010
-        monthsToDisplay = 12;
-        mFrom = 1;
-        yFrom = atoi (context.args[1].c_str ());
+
+      // "January" etc.
+      else if (autoComplete (lowerCase (*arg), monthNames, matches) == 1)
+      {
+        argMonth = Date::monthOfYear (matches[0]);
+        if (argMonth == -1)
+          throw std::string ("Argument '") + *arg + "' is not a valid month.";
       }
+
+      else
+        throw std::string ("Could not recognize argument '") + *arg + "'.";
     }
-    else if (numberOfArgs == 3) {
-      if (context.args[2] == "y") {
-        // task cal due y
-        monthsToDisplay = 12;
-        getpendingdate = true;
-      }
-      else {
-        // task cal 8 2010
-        monthsToDisplay = monthsPerLine;
-        mFrom = atoi (context.args[1].c_str ());
-        yFrom = atoi (context.args[2].c_str ());
-      }
-    }
-    else if (numberOfArgs == 4) {
-      // task cal 8 2010 y
+
+    // Supported combinations:
+    //
+    //   Command line  monthsToDisplay  mFrom  yFrom  getpendingdate
+    //   ------------  ---------------  -----  -----  --------------
+    //   cal             monthsPerLine  today  today           false
+    //   cal y                      12  today  today           false
+    //   cal due         monthsPerLine  today  today            true
+    //   cal YYYY                   12      1    arg           false
+    //   cal due y                  12  today  today            true
+    //   cal MM YYYY     monthsPerLine    arg    arg           false
+    //   cal MM YYYY y              12    arg    arg           false
+
+    if (argWholeYear || (argYear && !argMonth && !argWholeYear))
       monthsToDisplay = 12;
-      mFrom = atoi (context.args[1].c_str ());
-      yFrom = atoi (context.args[2].c_str ());
-    }
 
+    if (!argMonth && argYear)
+      mFrom = 1;
+    else if (argMonth && argYear)
+      mFrom = argMonth;
+
+    if (argYear)
+      yFrom = argYear;
+
+    // Now begin the data subset and rendering.
     int countDueDates = 0;
     if (getpendingdate == true) {
       // Find the oldest pending due date.
@@ -2096,7 +2154,8 @@ int handleReportCalendar (std::string &outs)
       {
         if (task->getStatus () == Task::pending)
         {
-          if (task->has ("due"))
+          if (task->has ("due") &&
+              !task->hasTag ("nocal"))
           {
             ++countDueDates;
             Date d (atoi (task->get ("due").c_str ()));
@@ -2119,7 +2178,7 @@ int handleReportCalendar (std::string &outs)
     int details_mFrom = mFrom;
 
     std::stringstream out;
-    out << std::endl;
+    out << "\n";
 
     while (yFrom < yTo || (yFrom == yTo && mFrom <= mTo))
     {
@@ -2163,10 +2222,10 @@ int handleReportCalendar (std::string &outs)
         }
       }
 
-      out << std::endl
+      out << "\n"
           << optionalBlankLine ()
           << renderMonths (mFrom, yFrom, today, tasks, monthsPerLine)
-          << std::endl;
+          << "\n";
 
       mFrom += monthsPerLine;
       if (mFrom > 12)
@@ -2202,7 +2261,7 @@ int handleReportCalendar (std::string &outs)
           << color_weeknumber.colorize ("weeknumber")
           << "."
           << optionalBlankLine ()
-          << std::endl;
+          << "\n";
 
     if (context.config.get ("calendar.details") == "full" || context.config.get ("calendar.holidays") == "full")
     {
@@ -2301,7 +2360,7 @@ int handleReportCalendar (std::string &outs)
 
         out << optionalBlankLine ()
             << holTable.render ()
-            << std::endl;
+            << "\n";
       }
     }
 
@@ -2313,7 +2372,7 @@ int handleReportCalendar (std::string &outs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int handleReportStats (std::string &outs)
+int handleReportStats (std::string& outs)
 {
   int rc = 0;
 
@@ -2386,7 +2445,7 @@ int handleReportStats (std::string &outs)
       }
 
       if (it->getStatus () == Task::pending)
-        daysPending += (now - entry) / 86400.0;
+        daysPending += (now.toEpoch () - entry) / 86400.0;
 
       descLength += it->get ("description").length ();
 
@@ -2537,6 +2596,14 @@ int handleReportStats (std::string &outs)
       table.addCell (row, 1, value.str ());
     }
 
+    // If an alternating row color is specified, notify the table.
+    if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+    {
+      Color alternate (context.config.get ("color.alternate"));
+      if (alternate.nontrivial ())
+        table.setTableAlternateColor (alternate);
+    }
+
     out << optionalBlankLine ()
         << table.render ()
         << optionalBlankLine ();
@@ -2579,6 +2646,22 @@ void gatherNextTasks (std::vector <Task>& tasks)
     }
   }
 
+  // blocking, not blocked
+  foreach (task, tasks)
+  {
+    if (dependencyIsBlocking (*task) &&
+        ! dependencyIsBlocked (*task))
+    {
+      std::string project = task->get ("project");
+      if (countByProject[project] < limit && matching.find (task->id) == matching.end ())
+      {
+        ++countByProject[project];
+        matching[task->id] = true;
+        filtered.push_back (*task);
+      }
+    }
+  }
+
   // due:*, pri:H
   foreach (task, tasks)
   {
@@ -2745,7 +2828,10 @@ std::string getFullDescription (Task& task, const std::string& report)
         desc = "+" + desc;
       Att anno (annotations.back());
       Date dt (atoi (anno.name ().substr (11).c_str ()));
-      std::string when = dt.toString (context.config.get ("dateformat"));
+      std::string format = context.config.get ("dateformat.annotation");
+      if (format == "")
+        format = context.config.get ("dateformat");
+      std::string when = dt.toString (format);
       desc += "\n" + when + " " + anno.value ();
     }
     else
@@ -2777,3 +2863,87 @@ std::string getDueDate (Task& task, const std::string& format)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
+std::string onProjectChange (Task& task, bool scope /* = true */)
+{
+  std::stringstream msg;
+  std::string project = task.get ("project");
+
+  if (project != "")
+  {
+    if (scope)
+      msg << "The project '"
+          << project
+          << "' has changed.  ";
+
+    // Count pending and done tasks, for this project.
+    int count_pending = 0;
+    int count_done = 0;
+
+    std::vector <Task> all;
+    Filter filter;
+    context.tdb.load (all, filter);
+
+    countTasks (all,                           project, task.get ("uuid"), count_pending, count_done);
+    countTasks (context.tdb.getAllNew (),      project, "nope",            count_pending, count_done);
+    countTasks (context.tdb.getAllModified (), project, "nope",            count_pending, count_done);
+
+    int percentage = 0;
+    if (count_done + count_pending > 0)
+      percentage = (count_done * 100 / (count_done + count_pending));
+
+    msg << "Project '"
+        << project
+        << "' is "
+        << percentage
+        << "% complete ("
+        << count_pending
+        << " of "
+        << (count_pending + count_done)
+        << " tasks remaining).\n";
+  }
+
+  return msg.str ();
+}
+
+///////////////////////////////////////////////////////////////////////////////
+std::string onProjectChange (Task& task1, Task& task2)
+{
+  std::string messages = onProjectChange (task1);
+  messages            += onProjectChange (task2);
+
+  return messages;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+static void countTasks (
+  const std::vector <Task>& all,
+  const std::string& project,
+  const std::string& skip,
+  int& count_pending,
+  int& count_done)
+{
+  std::vector <Task>::const_iterator it;
+  for (it = all.begin (); it != all.end (); ++it)
+  {
+    if (it->get ("project") == project &&
+        it->get ("uuid")    != skip)
+    {
+      switch (it->getStatus ())
+      {
+      case Task::pending:
+      case Task::waiting:
+        ++count_pending;
+        break;
+
+      case Task::completed:
+        ++count_done;
+        break;
+
+      default:
+        break;
+      }
+    }
+  }
+}
+
+///////////////////////////////////////////////////////////////////////////////
