@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////
-// task - a command line task list manager.
+// taskwarrior - a command line task list manager.
 //
 // Copyright 2006 - 2010, Paul Beckingham.
 // All rights reserved.
@@ -195,8 +195,8 @@ int autoComplete (
       }
 
       // Maintain a list of partial matches.
-      if (length <= item->length () &&
-          partial == item->substr (0, length))
+      else if (length <= item->length () &&
+               partial == item->substr (0, length))
         matches.push_back (*item);
     }
   }
@@ -368,29 +368,69 @@ std::string taskDifferences (const Task& before, const Task& after)
   foreach (name, beforeOnly)
     out << "  - "
         << *name
-        << " will be deleted\n";
+        << " will be deleted.\n";
 
   foreach (name, afterOnly)
-    out << "  - "
-        << *name
-        << " will be set to '"
-        << renderAttribute (*name, after.get (*name))
-        << "'\n";
+  {
+    if (*name == "depends")
+    {
+      std::vector <int> deps_after;
+      after.getDependencies (deps_after);
+      std::string to;
+      join (to, ",", deps_after);
+
+      out << "  - "
+          << *name
+          << " will be set to '"
+          << to
+          << "'.\n";
+    }
+    else
+      out << "  - "
+          << *name
+          << " will be set to '"
+          << renderAttribute (*name, after.get (*name))
+          << "'.\n";
+  }
 
   foreach (name, beforeAtts)
     if (*name              != "uuid" &&
         before.get (*name) != after.get (*name))
-      out << "  - "
-          << *name
-          << " will be changed from '"
-          << renderAttribute (*name, before.get (*name))
-          << "' to '"
-          << renderAttribute (*name, after.get (*name))
-          << "'\n";
+    {
+      if (*name == "depends")
+      {
+        std::vector <int> deps_before;
+        before.getDependencies (deps_before);
+        std::string from;
+        join (from, ",", deps_before);
+
+        std::vector <int> deps_after;
+        after.getDependencies (deps_after);
+        std::string to;
+        join (to, ",", deps_after);
+
+        out << "  - "
+            << *name
+            << " will be changed from '"
+            << from
+            << "' to '"
+            << to
+            << "'.\n";
+      }
+      else
+        out << "  - "
+            << *name
+            << " will be changed from '"
+            << renderAttribute (*name, before.get (*name))
+            << "' to '"
+            << renderAttribute (*name, after.get (*name))
+            << "'.\n";
+    }
+
 
   // Shouldn't just say nothing.
   if (out.str ().length () == 0)
-    out << "  - No changes will be made\n";
+    out << "  - No changes will be made.\n";
 
   return out.str ();
 }
